import json
import os

from django.core.files.base import ContentFile
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import RequestFactory, TestCase

from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection, TemporaryFile
from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.views.temporary_file_upload_api import (
    UploadTemporaryFilesView,
)
from cradmin_legacy.tests.helpers import create_user


class TestUploadTemporaryFilesView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.testuser = create_user("testuser")

    def test_post_single_file(self):
        request = self.factory.post("/test", {"file": SimpleUploadedFile("testfile1.txt", b"Test1")})
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertIsNotNone(responsedata["collectionid"])

        collectionid = responsedata["collectionid"]
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEqual(collection.files.count(), 1)
        uploadedfile = collection.files.first()
        self.assertEqual(uploadedfile.filename, "testfile1.txt")
        self.assertNotIn("testfile1.txt", uploadedfile.file.name)
        self.assertTrue(uploadedfile.file.name.startswith(f"cradmin_temporaryfileuploadstore/{collection.id}/"))
        self.assertEqual(uploadedfile.file.read(), b"Test1")

        self.assertEqual(len(responsedata["temporaryfiles"]), 1)
        self.assertEqual(responsedata["temporaryfiles"][0]["filename"], "testfile1.txt")

    def test_post_multiple_files(self):
        request = self.factory.post(
            "/test",
            {
                "file": [
                    SimpleUploadedFile("testfile1.txt", b"Test1"),
                    SimpleUploadedFile("testfile2.txt", b"Test2"),
                ]
            },
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertIsNotNone(responsedata["collectionid"])

        collectionid = responsedata["collectionid"]
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEqual(collection.files.count(), 2)
        self.assertEqual(len(responsedata["temporaryfiles"]), 2)
        self.assertEqual(responsedata["temporaryfiles"][0]["filename"], "testfile1.txt")
        self.assertEqual(responsedata["temporaryfiles"][1]["filename"], "testfile2.txt")

    def test_post_multiple_requests_for_same_collection(self):
        request1 = self.factory.post("/test", {"file": SimpleUploadedFile("testfile1.txt", b"Test1")})
        request1.user = self.testuser
        response1 = UploadTemporaryFilesView.as_view()(request1)
        self.assertEqual(response1.status_code, 200)
        response1data = json.loads(response1.content.decode("utf-8"))
        self.assertIsNotNone(response1data["collectionid"])
        collectionid = response1data["collectionid"]
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEqual(collection.files.count(), 1)

        request2 = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testfile2.txt", b"Test2"), "collectionid": collectionid}
        )
        request2.user = self.testuser
        response2 = UploadTemporaryFilesView.as_view()(request2)
        self.assertEqual(response2.status_code, 200)
        response2data = json.loads(response2.content.decode("utf-8"))
        self.assertEqual(collectionid, response2data["collectionid"])
        self.assertEqual(collection.files.count(), 2)

    def test_post_multiple_files_singlefile_mode(self):
        request1 = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testfile1.txt", b"Test1"), "singlemode": "true"}
        )
        request1.user = self.testuser
        response1 = UploadTemporaryFilesView.as_view()(request1)
        self.assertEqual(response1.status_code, 200)
        response1data = json.loads(response1.content.decode("utf-8"))
        self.assertIsNotNone(response1data["collectionid"])
        collectionid = response1data["collectionid"]
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEqual(collection.files.count(), 1)
        uploadedfile1 = collection.files.first()
        uploadedfile1_physical_file_path = uploadedfile1.file.path
        self.assertTrue(os.path.exists(uploadedfile1_physical_file_path))

        request2 = self.factory.post(
            "/test",
            {"file": SimpleUploadedFile("testfile2.txt", b"Test2"), "collectionid": collectionid, "singlemode": "true"},
        )
        request2.user = self.testuser
        response2 = UploadTemporaryFilesView.as_view()(request2)
        self.assertEqual(response2.status_code, 200)
        response2data = json.loads(response2.content.decode("utf-8"))
        self.assertEqual(collectionid, response2data["collectionid"])
        self.assertEqual(collection.files.count(), 1)
        uploadedfile2 = collection.files.first()
        self.assertEqual(uploadedfile2.filename, "testfile2.txt")
        self.assertEqual(uploadedfile2.file.read(), b"Test2")
        self.assertFalse(os.path.exists(uploadedfile1_physical_file_path))

    def test_post_accept_ok(self):
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testfile1.txt", b"Test1"), "accept": "text/plain,application/pdf"}
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))
        collectionid = responsedata["collectionid"]

        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEqual(collection.accept, "text/plain,application/pdf")
        self.assertEqual(collection.files.count(), 1)
        uploadedfile = collection.files.first()
        self.assertEqual(uploadedfile.mimetype, "text/plain")

        self.assertEqual(responsedata["temporaryfiles"][0]["mimetype"], "text/plain")

    def test_post_accept_invalid_single_file(self):
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testfile1.txt", b"Test1"), "accept": "application/pdf"}
        )
        request.user = self.testuser
        self.assertEqual(TemporaryFileCollection.objects.count(), 0)
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 400)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(TemporaryFileCollection.objects.count(), 0)
        self.assertEqual(responsedata["file"][0]["code"], "unsupported_mimetype")
        self.assertEqual(responsedata["file"][0]["message"], "testfile1.txt: Unsupported filetype.")

    def test_post_accept_invalid_existing_collection(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile.txt")
        temporaryfile.save()

        request = self.factory.post(
            "/test",
            {
                "collectionid": collection.id,
                "file": SimpleUploadedFile("testfile1.txt", b"Test1"),
                "accept": "application/pdf",
            },
        )
        request.user = self.testuser
        self.assertEqual(TemporaryFileCollection.objects.count(), 1)
        self.assertEqual(collection.files.count(), 1)
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 400)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(TemporaryFileCollection.objects.count(), 1)
        self.assertEqual(collection.files.count(), 1)
        self.assertEqual(responsedata["file"][0]["code"], "unsupported_mimetype")
        self.assertEqual(responsedata["file"][0]["message"], "testfile1.txt: Unsupported filetype.")

    def test_post_accept_invalid_multiple_files(self):
        request = self.factory.post(
            "/test",
            {
                "file": [
                    SimpleUploadedFile("testfile1.pdf", b"Test1"),
                    SimpleUploadedFile("testfile2.txt", b"Test2"),
                ],
                "accept": "application/pdf",
            },
        )
        request.user = self.testuser
        self.assertEqual(TemporaryFileCollection.objects.count(), 0)
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 400)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(TemporaryFileCollection.objects.count(), 0)
        self.assertEqual(responsedata["file"][0]["code"], "unsupported_mimetype")
        self.assertEqual(responsedata["file"][0]["message"], "testfile2.txt: Unsupported filetype.")

    def test_post_max_filename_length_not_truncated(self):
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("abc.txt", b"Testcontent"), "max_filename_length": "7"}
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))
        collectionid = responsedata["collectionid"]

        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEqual(collection.files.first().filename, "abc.txt")
        self.assertEqual(responsedata["temporaryfiles"][0]["filename"], "abc.txt")

    def test_post_max_filename_length_truncated(self):
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("abc.txt", b"Testcontent"), "max_filename_length": "6"}
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))
        collectionid = responsedata["collectionid"]

        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEqual(collection.files.first().filename, "bc.txt")
        self.assertEqual(responsedata["temporaryfiles"][0]["filename"], "bc.txt")

    def test_post_unique_filenames(self):
        request = self.factory.post(
            "/test",
            {
                "file": [
                    SimpleUploadedFile("abc.txt", b"Testcontent1"),
                    SimpleUploadedFile("abc.txt", b"Testcontent2"),
                ],
                "unique_filenames": True,
            },
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))
        collectionid = responsedata["collectionid"]
        collection = TemporaryFileCollection.objects.get(id=collectionid)

        files_by_content = {}
        for temporaryfile in collection.files.all():
            files_by_content[temporaryfile.file.read()] = temporaryfile
        self.assertEqual(files_by_content[b"Testcontent1"].filename, "abc.txt")
        self.assertNotEqual(files_by_content[b"Testcontent2"].filename, "abc.txt")
        self.assertTrue(files_by_content[b"Testcontent2"].filename.endswith("abc.txt"))

        self.assertEqual(responsedata["temporaryfiles"][0]["filename"], "abc.txt")
        self.assertNotEqual(responsedata["temporaryfiles"][1]["filename"], "abc.txt")
        self.assertTrue(responsedata["temporaryfiles"][1]["filename"].endswith("abc.txt"))

    def test_post_unique_filenames_and_max_filename_length(self):
        request = self.factory.post(
            "/test",
            {
                "file": [
                    SimpleUploadedFile("t" * 50, b"Testcontent1"),
                    SimpleUploadedFile("t" * 50, b"Testcontent2"),
                ],
                "unique_filenames": True,
                "max_filename_length": 45,
            },
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))
        collectionid = responsedata["collectionid"]
        collection = TemporaryFileCollection.objects.get(id=collectionid)

        files_by_content = {}
        for temporaryfile in collection.files.all():
            files_by_content[temporaryfile.file.read()] = temporaryfile
        self.assertEqual(files_by_content[b"Testcontent1"].filename, "ttttttttttttttttttttt...ttttttttttttttttttttt")
        self.assertNotEqual(files_by_content[b"Testcontent2"].filename, "ttttttttttttttttttttt...ttttttttttttttttttttt")
        self.assertTrue(files_by_content[b"Testcontent2"].filename.endswith("-tttttttt"))

    def test_post_max_filesize_bytes_size_below_ok(self):
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testname.txt", b"Testcontent"), "max_filesize_bytes": 100}
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))
        collectionid = responsedata["collectionid"]
        collection = TemporaryFileCollection.objects.get(id=collectionid)
        self.assertEqual(collection.files.count(), 1)

    def test_post_max_filesize_bytes_size_above_fails(self):
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testname.txt", b"Testcontent"), "max_filesize_bytes": 1}
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 400)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual("max_filesize_bytes_exceeded", responsedata["file"][0]["code"])
        self.assertFalse(TemporaryFileCollection.objects.exists())

    def test_post_form_invalid_no_file(self):
        request = self.factory.post("/test")
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 400)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(responsedata["file"][0]["code"], "required")
        self.assertFalse(TemporaryFileCollection.objects.exists())

    def test_post_form_invalid_collectionid_not_number(self):
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testfile1.txt", b"Test1"), "collectionid": "invalid"}
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 400)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(responsedata["collectionid"][0]["code"], "invalid")

    def test_post_form_collectionid_does_not_exist(self):
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testfile1.txt", b"Test1"), "collectionid": "10001"}
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 404)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(responsedata["collectionid"][0]["code"], "doesnotexist")

    def test_post_form_collection_not_owned_by_user(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        request = self.factory.post(
            "/test", {"file": SimpleUploadedFile("testfile1.txt", b"Test1"), "collectionid": collection.id}
        )
        request.user = create_user("otheruser")
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 404)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(responsedata["collectionid"][0]["code"], "doesnotexist")

    def test_delete_single_file(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile.txt")
        temporaryfile.file.save("testfile.txt", ContentFile("testcontent"))

        request = self.factory.delete(
            "/test", json.dumps({"collectionid": collection.id, "temporaryfileid": temporaryfile.id})
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        responsedata = json.loads(response.content.decode("utf-8"))

        self.assertEqual(responsedata["collectionid"], collection.id)
        self.assertEqual(responsedata["temporaryfileid"], temporaryfile.id)

        self.assertTrue(TemporaryFileCollection.objects.filter(id=collection.id).exists())
        self.assertFalse(TemporaryFile.objects.filter(id=temporaryfile.id).exists())

    def test_delete_invalid_json_reqeust(self):
        request = self.factory.delete("/test", data="invalid")
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 400)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(responsedata["errormessage"], "Invalid JSON data in the request body.")

    def test_delete_form_collectionid_does_not_exist(self):
        request = self.factory.delete(
            "/test",
            data=json.dumps(
                {
                    "collectionid": "10001",
                    "temporaryfileid": "1",  # NOTE: This is ignored since we never get to the code looking it up
                }
            ),
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 404)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(responsedata["collectionid"][0]["code"], "doesnotexist")

    def test_delete_form_collection_not_owned_by_user(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        request = self.factory.delete(
            "/test",
            json.dumps(
                {
                    "collectionid": collection.id,
                    "temporaryfileid": "1",  # NOTE: This is ignored since we never get to the code looking it up
                }
            ),
        )
        request.user = create_user("otheruser")
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 404)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(responsedata["collectionid"][0]["code"], "doesnotexist")

    def test_delete_form_temporaryfileid_does_not_exist(self):
        collection = TemporaryFileCollection.objects.create(user=self.testuser)
        request = self.factory.delete(
            "/test", data=json.dumps({"collectionid": collection.id, "temporaryfileid": 10001})
        )
        request.user = self.testuser
        response = UploadTemporaryFilesView.as_view()(request)
        self.assertEqual(response.status_code, 404)
        responsedata = json.loads(response.content.decode("utf-8"))
        self.assertEqual(responsedata["temporaryfileid"][0]["code"], "doesnotexist")



from django.urls import path

from cradmin_legacy.apps.cradmin_activate_account.views.activate import ActivateAccountView

urlpatterns = [
    path("activate/<str:token>", ActivateAccountView.as_view(), name="cradmin-activate-account-activate"),
]



from django.urls import path
from django.contrib.auth.decorators import login_required

from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.views.temporary_file_upload_api import (
    UploadTemporaryFilesView,
)

urlpatterns = [
    path(
        "temporary_file_upload_api",
        login_required(UploadTemporaryFilesView.as_view()),
        name="cradmin_temporary_file_upload_api",
    ),
]



from django.urls import path

from cradmin_legacy.apps.cradmin_resetpassword.views.begin import BeginPasswordResetView
from cradmin_legacy.apps.cradmin_resetpassword.views.email_sent import EmailSentView
from cradmin_legacy.apps.cradmin_resetpassword.views.reset import ResetPasswordView

urlpatterns = [
    path("begin", BeginPasswordResetView.as_view(), name="cradmin-resetpassword-begin"),
    path("email-sent", EmailSentView.as_view(), name="cradmin-resetpassword-email-sent"),
    path("reset/<str:token>", ResetPasswordView.as_view(), name="cradmin-resetpassword-reset"),
]



from django.test import TestCase
from django.urls import reverse
import htmls
from cradmin_legacy.apps.cradmin_authenticate.tests.cradmin_authenticate_testapp.models import EmailUser
from django.test.utils import override_settings

from cradmin_legacy.tests.views.helpers import create_testuser


class TestUsernameLogin(TestCase):
    def setUp(self):
        self.testuser = create_testuser(username="testuser")
        self.url = reverse("cradmin-authenticate-login")

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertEqual(selector.one("h1").alltext_normalized, "Sign in")

    def test_login_ok(self):
        response = self.client.post(self.url, {"username": "testuser", "password": "test"})
        self.assertEqual(response.status_code, 302)

    def test_login_invalid(self):
        response = self.client.post(self.url, {"username": "testuser", "password": "invalid"})
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertIn(
            "Your username and password didn't match",
            selector.one("#cradmin_authenticate_login_form").alltext_normalized,
        )


class TestEmailLogin(TestCase):
    def setUp(self):
        self.testuser = EmailUser(email="testuser@example.com")
        self.testuser.set_password("test")
        self.testuser.save()
        self.url = reverse("cradmin-authenticate-login")

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertEqual(selector.one("h1").alltext_normalized, "Sign in")

    def test_login_ok(self):
        with self.settings(LOGIN_REDIRECT_URL="/login/redirect"):
            response = self.client.post(self.url, {"email": "testuser@example.com", "password": "test"})
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response["location"], "/login/redirect")

    def test_login_next(self):
        response = self.client.post(
            f"{self.url}?next=/next", {"email": "testuser@example.com", "password": "test"}
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response["location"], "/next")

    def test_login_invalid(self):
        response = self.client.post(self.url, {"email": "testuser@example.com", "password": "invalid"})
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertIn(
            "Your email and password didn't match", selector.one("#cradmin_authenticate_login_form").alltext_normalized
        )


TestEmailLogin = override_settings(
    AUTH_USER_MODEL="cradmin_authenticate_testapp.EmailUser", CRADMIN_LEGACY_USE_EMAIL_AUTH=True
)(TestEmailLogin)



from django.http import HttpResponse
from django.views.generic import View

from cradmin_legacy.apps.cradmin_email import emailutils


class DemoEmail(emailutils.AbstractEmail):
    subject_template = "cradmin_email/cradmin_email_send_testmail/subject.django.txt"
    html_message_template = "cradmin_email/cradmin_email_send_testmail/html_message.django.html"

    def get_context_data(self):
        context = super().get_context_data()
        context["name"] = "Test Name"
        return context


class EmailDesignView(View):
    def get(self, request, format="html"):
        email = DemoEmail()
        if format == "plaintext":
            return HttpResponse(email.render_plaintext_message(), content_type="text/plain")
        else:
            return HttpResponse(email.render_html_message())



import htmls
from cradmin_legacy.python2_compatibility import mock
from django.test import TestCase, RequestFactory

from cradmin_legacy.tests.viewhelpers.cradmin_viewhelpers_testapp.models import TestModel
from cradmin_legacy.viewhelpers import objecttable
from cradmin_legacy.cradmin_legacy_testapp import models as testmodels


class TestColumn(TestCase):
    def setUp(self):
        class TestColSubclass(objecttable.Column):
            modelfield = "testfield"

            def __init__(self, **kwargs):
                super().__init__(**kwargs)

        self.model_testobject = TestModel()
        view = mock.MagicMock()
        view.model = self.model_testobject
        self.column_subclass = TestColSubclass(view=view, columnindex=0)

    def test_get_header_not_implemented(self):
        test = objecttable.Column(view=None, columnindex=0)

        with self.assertRaises(NotImplementedError):
            test.get_header()

    def test_get_header(self):
        self.assertEqual("Test Value", self.column_subclass.get_header())

    def test_render_value(self):
        self.model_testobject.testfield = "test_value"
        self.assertEqual("test_value", self.column_subclass.render_value(self.model_testobject))

    # check that you get an exception when running render_value without having modelfield..
    def test_render_value_not_implemented(self):
        col = objecttable.Column(view=None, columnindex=0)

        with self.assertRaises(NotImplementedError):
            col.render_value(None)

    # check that you get an exception when running render_cell_content without overriding with subclass..
    def test_render_cell_not_implemented(self):
        col = objecttable.Column(view=mock.MagicMock(), columnindex=0)

        with self.assertRaises(NotImplementedError):
            col.render_cell_content(None)


class TestPlainTextColumn(TestCase):
    def setUp(self):
        class TestColSubclass(objecttable.PlainTextColumn):
            modelfield = "testfield"

            def __init__(self, **kwargs):
                super().__init__(**kwargs)

        self.model_testobject = TestModel()
        view = mock.MagicMock()
        view.model = self.model_testobject
        self.column_subclass = TestColSubclass(view=view, columnindex=0)

    def test_render_cell(self):
        self.model_testobject.testfield = "test_value"
        self.assertEqual(
            '<span class="objecttable-cellvalue">test_value</span>',
            self.column_subclass.render_cell_content(self.model_testobject).strip(),
        )


class TestSingleActionColumn(TestCase):
    def setUp(self):
        class TestIncompleteColSubclass(objecttable.SingleActionColumn):
            modelfield = "testfield"

            def __init__(self, **kwargs):
                super().__init__(**kwargs)

        class TestColSubclass(objecttable.SingleActionColumn):
            modelfield = "testfield"

            def __init__(self, **kwargs):
                super().__init__(**kwargs)

            def get_actionurl(self, obj):
                return f"www.example.com/{obj.testfield}"

        self.model_testobject = TestModel(testfield="test_value")
        view = mock.MagicMock()
        view.model = self.model_testobject
        self.column_subclass = TestColSubclass(view=view, columnindex=0)
        self.column_subclass_incomplete = TestIncompleteColSubclass(view=view, columnindex=0)

    def test_get_actionurl_ExceptionRaised(self):
        with self.assertRaises(NotImplementedError):
            self.column_subclass_incomplete.get_actionurl(self.model_testobject)

    def test_render_cell_ExceptionRaised(self):
        with self.assertRaises(NotImplementedError):
            self.column_subclass_incomplete.render_cell_content(self.model_testobject)

    def test_render_cell(self):
        expected = '<a href="www.example.com/test_value" class="objecttable-cellvalue-link">test_value</a>'
        self.assertEqual(self.column_subclass.render_cell_content(self.model_testobject), expected)


class TestMultiActionColumn(TestCase):
    def setUp(self):
        class TestIncompleteColSubclass(objecttable.MultiActionColumn):
            modelfield = "testfield"

            def __init__(self, **kwargs):
                super().__init__(**kwargs)

        class TestColSubclass(objecttable.MultiActionColumn):
            modelfield = "testfield"

            def __init__(self, **kwargs):
                super().__init__(**kwargs)

            def get_buttons(self, obj):
                return [
                    objecttable.Button(label="Btn1", url="www.example.com/btn1"),
                    objecttable.Button(label="Btn2", url="www.example.com/btn2"),
                ]

        self.model_testobject = TestModel(testfield="test_value")
        view = mock.MagicMock()
        view.model = self.model_testobject
        self.column_subclass = TestColSubclass(view=view, columnindex=0)
        self.column_subclass_incomplete = TestIncompleteColSubclass(view=view, columnindex=0)

    def test_get_buttons_ExceptionRaised(self):
        with self.assertRaises(NotImplementedError):
            self.column_subclass_incomplete.get_buttons(self.model_testobject)

    def test_render_cell_ExceptionRaised(self):
        with self.assertRaises(NotImplementedError):
            self.column_subclass_incomplete.render_cell_content(self.model_testobject)

    def test_render_cell(self):
        result = self.column_subclass.render_cell_content(self.model_testobject)
        selector = htmls.S(result)
        self.assertEqual(selector.one("p.objecttable-cellvalue").alltext_normalized, "test_value")
        self.assertTrue(selector.exists("p.objecttable-cellbuttons"))
        self.assertEqual(selector.count("a"), 2)


class TestButton(TestCase):
    def test_render_with_icon_and_class(self):
        btn = objecttable.Button(
            label="My Btn",
            url="www.example.com/mybtnurl",
            buttonclass="btn btn-danger btn-sm",
            icon="glyphicon glyphicon-shopping-cart",
        )
        selector = htmls.S(btn.render())
        self.assertEqual(selector.one("a")["href"], "www.example.com/mybtnurl")
        self.assertEqual(selector.one("a")["class"], "btn btn-danger btn-sm")
        self.assertEqual(selector.one("a").alltext_normalized, "My Btn")
        self.assertEqual(selector.one("a span")["class"], "glyphicon glyphicon-shopping-cart")

    def test_render_simple(self):
        btn = objecttable.Button(label="My Btn", url="www.example.com/mybtnurl")
        selector = htmls.S(btn.render())
        self.assertEqual(selector.one("a")["href"], "www.example.com/mybtnurl")
        self.assertEqual(selector.one("a")["class"], "btn btn-default btn-sm")
        self.assertEqual(selector.one("a").alltext_normalized, "My Btn")


class TestOrderingStringParser(TestCase):
    def test_parse_empty(self):
        orderingqueryarg = objecttable.OrderingStringParser("")
        self.assertEqual(len(orderingqueryarg.orderingdict), 0)

    def test_parse_single(self):
        orderingqueryarg = objecttable.OrderingStringParser("a3")
        self.assertEqual(len(orderingqueryarg.orderingdict), 1)
        self.assertTrue(orderingqueryarg.orderingdict[3])

    def test_parse_multi(self):
        orderingqueryarg = objecttable.OrderingStringParser("a3.d1")
        self.assertEqual(len(orderingqueryarg.orderingdict), 2)
        self.assertTrue(orderingqueryarg.orderingdict[3].order_ascending)
        self.assertFalse(orderingqueryarg.orderingdict[1].order_ascending)

    def test_remove_column(self):
        self.assertEqual(objecttable.OrderingStringParser("a3.d1.a6").remove_column(6), "a3.d1")
        self.assertEqual(objecttable.OrderingStringParser("a3.d1.a6").remove_column(1), "a3.a6")

    def test_remove_nonexisting_colum(self):
        self.assertEqual(objecttable.OrderingStringParser("").remove_column(2), "")
        self.assertEqual(objecttable.OrderingStringParser("a1").remove_column(2), "a1")
        self.assertEqual(objecttable.OrderingStringParser("a3.d1.a6").remove_column(2), "a3.d1.a6")

    def test_flip_existing_column(self):
        self.assertEqual(objecttable.OrderingStringParser("d1").flip_column(1), "a1")
        self.assertEqual(objecttable.OrderingStringParser("a3.d1.a6").flip_column(1), "a3.a1.a6")
        self.assertEqual(objecttable.OrderingStringParser("a3.d1.a6").flip_column(3), "d3.d1.a6")

    def test_flip_new_column(self):
        self.assertEqual(objecttable.OrderingStringParser("").flip_column(1), "a1")
        self.assertEqual(objecttable.OrderingStringParser("a3").flip_column(1), "a3.a1")


class TestObjectTableView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def _mock_request(self, request):
        request.cradmin_role = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()

    def test_empty(self):
        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.none()

        request = self.factory.get("/test")
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#cradmin_legacy_contentwrapper').prettyprint()
        self.assertFalse(selector.exists("#objecttableview-table"))
        self.assertEqual(selector.one("#objecttableview-no-items-message").alltext_normalized, "No some items")

    def test_empty_hide_search(self):
        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            searchfields = ["name"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.none()

        request = self.factory.get("/test")
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists(".cradmin-searchform"))

    def test_paginate_by_singlepage(self):
        testmodels.SomeItem.objects.bulk_create([testmodels.SomeItem(name=str(x)) for x in range(4)])

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            paginate_by = 4

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test")
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#cradmin_legacy_contentwrapper').prettyprint()
        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 4)
        self.assertFalse(selector.exists("#cradmin_legacy_contentwrapper .pager"))

    def test_paginate_by_firstpage(self):
        testmodels.SomeItem.objects.bulk_create([testmodels.SomeItem(name=str(x)) for x in range(5)])

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            paginate_by = 4

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test")
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 4)
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager"))
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager .previous.disabled"))
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager .next"))
        self.assertFalse(selector.exists("#cradmin_legacy_contentwrapper .pager .next.disabled"))

    def test_paginate_by_lastpage(self):
        testmodels.SomeItem.objects.bulk_create([testmodels.SomeItem(name=str(x)) for x in range(5)])

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            paginate_by = 4

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test", {"page": 2})
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 1)
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager"))
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager .previous"))
        self.assertFalse(selector.exists("#cradmin_legacy_contentwrapper .pager .previous.disabled"))
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager .next.disabled"))

    def test_paginate_by_middlepage(self):
        testmodels.SomeItem.objects.bulk_create([testmodels.SomeItem(name=str(x)) for x in range(9)])

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            paginate_by = 4

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test", {"page": 2})
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#cradmin_legacy_contentwrapper').prettyprint()
        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 4)
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager"))
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager .previous"))
        self.assertTrue(selector.exists("#cradmin_legacy_contentwrapper .pager .next"))
        self.assertFalse(selector.exists("#cradmin_legacy_contentwrapper .pager .previous.disabled"))
        self.assertFalse(selector.exists("#cradmin_legacy_contentwrapper .pager .next.disabled"))

    def test_render_single_simple_column(self):
        testmodels.SomeItem.objects.create(name="Item One")

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test")
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertEqual(selector.count("#objecttableview-table>thead>tr>th"), 1)
        self.assertEqual(
            selector.one("#objecttableview-table>thead>tr>th").alltext_normalized,
            "The name - Ordered descending - Click to order ascending",
        )

        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 1)
        self.assertEqual(selector.count("#objecttableview-table>tbody>tr>td"), 1)
        self.assertEqual(selector.one("#objecttableview-table>tbody>tr>td").alltext_normalized, "Item One")

    def test_render_multiple_simple_columns(self):
        testmodels.SomeItem.objects.create(name="Item One", somenumber=10)

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name", "somenumber"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test")
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertEqual(selector.count("#objecttableview-table>thead>tr>th"), 2)
        self.assertEqual(
            selector.one("#objecttableview-table>thead>tr>th:first-child").alltext_normalized,
            "The name - Ordered descending - Click to order ascending",
        )
        self.assertEqual(
            selector.one("#objecttableview-table>thead>tr>th:last-child").alltext_normalized,
            "Somenumber - Ordered descending - Click to order ascending",
        )

        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 1)
        self.assertEqual(selector.count("#objecttableview-table>tbody>tr>td"), 2)
        self.assertEqual(selector.one("#objecttableview-table>tbody>tr>td:first-child").alltext_normalized, "Item One")
        self.assertEqual(selector.one("#objecttableview-table>tbody>tr>td:last-child").alltext_normalized, "10")

    def test_render_order_ascending_singlecolumn(self):
        testmodels.SomeItem.objects.create(name="Item A")
        testmodels.SomeItem.objects.create(name="Item B")
        testmodels.SomeItem.objects.create(name="Item C")

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test", {"ordering": "a0"})
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 3)
        self.assertEqual(selector.one("#objecttableview-table>tbody>tr:first-child>td").alltext_normalized, "Item A")
        self.assertEqual(selector.one("#objecttableview-table>tbody>tr:last-child>td").alltext_normalized, "Item C")

        self.assertEqual(
            selector.one("#objecttableview-table>thead>tr>th").alltext_normalized,
            "The name - Ordered ascending - Click to order descending",
        )

    def test_render_order_descending_column(self):
        testmodels.SomeItem.objects.create(name="Item A")
        testmodels.SomeItem.objects.create(name="Item B")
        testmodels.SomeItem.objects.create(name="Item C")

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test", {"ordering": "d0"})
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#cradmin_legacy_contentwrapper thead').prettyprint()

        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 3)
        self.assertEqual(selector.one("#objecttableview-table>tbody>tr:first-child>td").alltext_normalized, "Item C")
        self.assertEqual(selector.one("#objecttableview-table>tbody>tr:last-child>td").alltext_normalized, "Item A")

        self.assertEqual(
            selector.one("#objecttableview-table>thead>tr>th").alltext_normalized,
            "The name - Ordered descending - Click to order ascending",
        )

    def test_render_order_multicolumn(self):
        testmodels.SomeItem.objects.create(name="Item A", somenumber=1)
        testmodels.SomeItem.objects.create(name="Item B", somenumber=2)
        testmodels.SomeItem.objects.create(name="Item C", somenumber=2)

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name", "somenumber"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test", {"ordering": "a1.d0"})
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#cradmin_legacy_contentwrapper thead').prettyprint()

        (self.assertEqual(selector.count("#objecttableview-table>thead>tr>th"), 2),)
        self.assertEqual(
            selector.one("#objecttableview-table>thead>tr>th:first-child").alltext_normalized,
            "The name - Ordered descending - Click to order ascending - Ordering priority 2",
        )
        self.assertEqual(
            selector.one("#objecttableview-table>thead>tr>th:last-child").alltext_normalized,
            "Somenumber - Ordered ascending - Click to order descending - Ordering priority 1",
        )

        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 3)
        self.assertEqual(
            selector.one("#objecttableview-table>tbody>tr:first-child>td:first-child").alltext_normalized, "Item A"
        )
        self.assertEqual(
            selector.one("#objecttableview-table>tbody>tr:last-child>td:first-child").alltext_normalized, "Item B"
        )

    def test_render_search_nomatch(self):
        testmodels.SomeItem.objects.create(name="Item One")

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name"]
            searchfields = ["name"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test", {"search": "Nothing matches this"})
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 0)

    def test_render_search_match(self):
        testmodels.SomeItem.objects.create(name="Item One")
        testmodels.SomeItem.objects.create(name="Item Two")
        testmodels.SomeItem.objects.create(name="Item Three")

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name"]
            searchfields = ["name"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/test", {"search": "Item Two"})
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertEqual(selector.count("#objecttableview-table>tbody>tr"), 1)
        self.assertEqual(selector.one("#objecttableview-table>tbody>tr>td").alltext_normalized, "Item Two")

    def test_show_column_headers(self):
        testmodels.SomeItem.objects.create(name="Item One")

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name"]

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/")
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertTrue(selector.exists("#objecttableview-table>thead"))
        self.assertFalse(selector.exists("#objecttableview-table>thead.sr-only"))

    def test_hide_column_headers(self):
        testmodels.SomeItem.objects.create(name="Item One")

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ["name"]
            hide_column_headers = True

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get("/")
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertTrue(selector.exists("#objecttableview-table>thead"))
        self.assertTrue(selector.exists("#objecttableview-table>thead.sr-only"))



from django.urls import path

from cradmin_legacy.apps.cradmin_register_account.views.begin import BeginRegisterAccountView
from cradmin_legacy.apps.cradmin_register_account.views.email_sent import EmailSentView

urlpatterns = [
    path("begin", BeginRegisterAccountView.as_view(), name="cradmin-register-account-begin"),
    path("email-sent", EmailSentView.as_view(), name="cradmin-register-account-email-sent"),
]



import htmls
from django.core.files.base import ContentFile
from django.test import RequestFactory, TestCase
from model_bakery import baker

from cradmin_legacy import cradmin_testhelpers
from cradmin_legacy.apps.cradmin_imagearchive import cradminviews
from cradmin_legacy.apps.cradmin_imagearchive.models import ArchiveImage
from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models import (
    TemporaryFile,
    TemporaryFileCollection,
)
from cradmin_legacy.cradmin_legacy_testapp.models import TstRole
from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.tests.helpers import create_user

from .helpers import create_image


def _create_single_file_temporaryfilecollection(user, filename="testfile.png", image=None):
    if image:
        testimage = image
    else:
        testimage = create_image(200, 100)
    collection = TemporaryFileCollection.objects.create(user=user)
    temporaryfile = TemporaryFile(collection=collection, filename=filename)
    temporaryfile.file.save(filename, ContentFile(testimage))
    return collection


class TestUpdateView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.role = TstRole.objects.create()
        self.archiveimage = ArchiveImage.objects.create(
            role=self.role, name="Original name", description="Original description"
        )
        self.testimage = create_image(200, 100)
        self.archiveimage.image.save("testimage.png", ContentFile(self.testimage))
        self.testuser = create_user("testuser")

    def test_get_render(self):
        request = self.factory.get("/test")
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_role = self.role

        response = cradminviews.ArchiveImageUpdateView.as_view()(request, pk=self.archiveimage.pk)
        self.assertEqual(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists("textarea#id_description"))

    def test_post_ok(self):
        request = self.factory.post(
            "/test",
            {
                # 'name': 'Updated name',
                "description": "Updated description"
            },
        )
        request.cradmin_instance = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()
        request.cradmin_app.reverse_appurl.return_value = "/success"
        request.cradmin_role = self.role
        request._messages = mock.MagicMock()

        self.assertEqual(ArchiveImage.objects.count(), 1)
        response = cradminviews.ArchiveImageUpdateView.as_view()(request, pk=self.archiveimage.pk)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response["Location"], "/success")
        self.assertEqual(ArchiveImage.objects.count(), 1)
        updated_image = ArchiveImage.objects.first()
        self.assertEqual(updated_image.image.read(), self.testimage)
        # self.assertEqual(updated_image.name, 'Updated name')
        self.assertEqual(updated_image.description, "Updated description")


class TestArchiveImagesListView(TestCase, cradmin_testhelpers.TestCaseMixin):
    viewclass = cradminviews.ArchiveImagesListView

    def test_no_archive_images(self):
        mockresponse = self.mock_http200_getrequest_htmls(
            requestuser=create_user("testuser"), cradmin_role=baker.make("cradmin_legacy_testapp.TstRole")
        )
        self.assertEqual(
            mockresponse.selector.one("#objecttableview-no-items-message").alltext_normalized, "No archive images"
        )

    def test_single_archive_image_sanity(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        baker.make("cradmin_imagearchive.ArchiveImage", name="Test image", description="", role=testrole)
        mockresponse = self.mock_http200_getrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertFalse(mockresponse.selector.exists("#objecttableview-no-items-message"))
        self.assertEqual(mockresponse.selector.count("#objecttableview-table tbody tr"), 1)

    def test_only_owned_by_role(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        other_role = baker.make("cradmin_legacy_testapp.TstRole")
        baker.make("cradmin_imagearchive.ArchiveImage", name="Test image", description="", role=other_role)
        mockresponse = self.mock_http200_getrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertTrue(mockresponse.selector.exists("#objecttableview-no-items-message"))

    def test_render_single_archive_image_no_description(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        baker.make("cradmin_imagearchive.ArchiveImage", name="Test image", description="", role=testrole)
        mockresponse = self.mock_http200_getrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertEqual(
            mockresponse.selector.one(
                "#objecttableview-table tbody tr td:last-child .objecttable-cellvalue"
            ).alltext_normalized,
            "Test image",
        )
        self.assertEqual(
            mockresponse.selector.one("#objecttableview-table tbody tr td:last-child .btn-default").alltext_normalized,
            "Set a description",
        )

    def test_render_single_archive_image_has_description(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        baker.make(
            "cradmin_imagearchive.ArchiveImage", name="Test image", description="Test description", role=testrole
        )
        mockresponse = self.mock_http200_getrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertEqual(
            mockresponse.selector.one(
                "#objecttableview-table tbody tr td:last-child .objecttable-cellvalue"
            ).alltext_normalized,
            "Test description",
        )
        self.assertEqual(
            mockresponse.selector.one("#objecttableview-table tbody tr td:last-child .btn-default").alltext_normalized,
            "Edit description",
        )

    def test_uploadform_get_render(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        mockresponse = self.mock_http200_getrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertEqual(mockresponse.response.status_code, 200)
        self.assertTrue(mockresponse.selector.exists("#cradmin_legacy_imagearchive_bulkadd_form"))
        self.assertTrue(mockresponse.selector.exists("input[type=hidden][name=filecollectionid]"))
        self.assertTrue(mockresponse.selector.exists("#div_id_filecollectionid"))

    def test_uploadform_post_no_filecollectionid(self):
        self.assertEqual(ArchiveImage.objects.count(), 0)
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        mockresponse = self.mock_http200_postrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertTrue(mockresponse.selector.exists("#div_id_filecollectionid.has-error"))
        self.assertIn(
            "You must upload at least one image.",
            mockresponse.selector.one("#div_id_filecollectionid").alltext_normalized,
        )

    def test_uploadform_post_single_image(self):
        testuser = create_user("testuser")
        testimage = create_image(200, 100)
        collection = TemporaryFileCollection.objects.create(user=testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile.png")
        temporaryfile.file.save("testfile.png", ContentFile(testimage))

        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        cradmin_app = mock.MagicMock()
        cradmin_app.reverse_appurl.return_value = "/success"
        self.assertEqual(ArchiveImage.objects.count(), 0)
        mockresponse = self.mock_http302_postrequest(
            requestuser=testuser,
            cradmin_role=testrole,
            cradmin_app=cradmin_app,
            requestkwargs={"data": {"filecollectionid": collection.id}},
        )

        self.assertEqual(mockresponse.response["Location"], "/success")
        self.assertEqual(ArchiveImage.objects.count(), 1)
        created_image = ArchiveImage.objects.first()
        created_image.image.open()
        self.assertEqual(created_image.image.read(), testimage)
        created_image.image.close()
        self.assertEqual(created_image.name, "testfile.png")
        self.assertEqual(created_image.description, "")

    def test_uploadform_post_deletes_collection(self):
        testuser = create_user("testuser")
        testimage = create_image(200, 100)
        collection = TemporaryFileCollection.objects.create(user=testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile.png")
        temporaryfile.file.save("testfile.png", ContentFile(testimage))

        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        self.mock_http302_postrequest(
            requestuser=testuser, cradmin_role=testrole, requestkwargs={"data": {"filecollectionid": collection.id}}
        )
        self.assertFalse(TemporaryFileCollection.objects.filter(id=collection.id).exists())

    def test_uploadform_post_multiple_images(self):
        testuser = create_user("testuser")
        testimage1 = create_image(200, 100)
        testimage2 = create_image(100, 100)
        collection = TemporaryFileCollection.objects.create(user=testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile1.png")
        temporaryfile.file.save("testfile1.png", ContentFile(testimage1))
        temporaryfile = TemporaryFile(collection=collection, filename="testfile2.png")
        temporaryfile.file.save("testfile2.png", ContentFile(testimage2))

        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        cradmin_app = mock.MagicMock()
        cradmin_app.reverse_appurl.return_value = "/success"

        self.assertEqual(ArchiveImage.objects.count(), 0)
        mockresponse = self.mock_http302_postrequest(
            requestuser=testuser,
            cradmin_role=testrole,
            cradmin_app=cradmin_app,
            requestkwargs={"data": {"filecollectionid": collection.id}},
        )
        self.assertEqual(mockresponse.response["Location"], "/success")
        self.assertEqual(ArchiveImage.objects.count(), 2)
        created_images = ArchiveImage.objects.order_by("name")

        for image in created_images:
            image.image.open()

        self.assertEqual(created_images[0].image.read(), testimage1)
        self.assertEqual(created_images[0].name, "testfile1.png")
        self.assertEqual(created_images[0].description, "")

        self.assertEqual(created_images[1].image.read(), testimage2)
        self.assertEqual(created_images[1].name, "testfile2.png")
        self.assertEqual(created_images[1].description, "")

        for image in created_images:
            image.image.close()

    def test_uploadform_post_sets_file_size(self):
        testuser = create_user("testuser")
        testimage = create_image(200, 100)
        collection = TemporaryFileCollection.objects.create(user=testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile.png")
        temporaryfile.file.save("testfile.png", ContentFile(testimage))

        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        self.mock_http302_postrequest(
            requestuser=testuser, cradmin_role=testrole, requestkwargs={"data": {"filecollectionid": collection.id}}
        )
        created_image = ArchiveImage.objects.first()
        self.assertEqual(len(testimage), created_image.file_size)


class TestArchiveImagesSingleSelectView(TestCase, cradmin_testhelpers.TestCaseMixin):
    viewclass = cradminviews.ArchiveImagesSingleSelectView

    def test_no_archive_images(self):
        mockresponse = self.mock_http200_getrequest_htmls(
            requestuser=create_user("testuser"), cradmin_role=baker.make("cradmin_legacy_testapp.TstRole")
        )
        self.assertEqual(
            mockresponse.selector.one("#objecttableview-no-items-message").alltext_normalized, "No archive images"
        )

    def test_single_archive_image_sanity(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        archiveimage = baker.make("cradmin_imagearchive.ArchiveImage", role=testrole)
        archiveimage.image.save("testimage.png", ContentFile(create_image(200, 100)))

        mockresponse = self.mock_http200_getrequest_htmls(
            requestuser=create_user("testuser"),
            cradmin_role=testrole,
            requestkwargs={"data": {"foreignkey_select_fieldid": "testfield"}},
        )
        self.assertFalse(mockresponse.selector.exists("#objecttableview-no-items-message"))
        self.assertEqual(mockresponse.selector.count("#objecttableview-table tbody tr"), 1)

    def test_only_owned_by_role(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        other_role = baker.make("cradmin_legacy_testapp.TstRole")
        baker.make("cradmin_imagearchive.ArchiveImage", name="Test image", description="", role=other_role)
        mockresponse = self.mock_http200_getrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertTrue(mockresponse.selector.exists("#objecttableview-no-items-message"))

    def test_render_single_archive_image_no_description(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        archiveimage = baker.make("cradmin_imagearchive.ArchiveImage", name="Test image", description="", role=testrole)
        archiveimage.image.save("testimage.png", ContentFile(create_image(200, 100)))
        mockresponse = self.mock_http200_getrequest_htmls(
            requestuser=create_user("testuser"),
            cradmin_role=testrole,
            requestkwargs={"data": {"foreignkey_select_fieldid": "testfield"}},
        )
        self.assertEqual(
            mockresponse.selector.one(
                "#objecttableview-table tbody tr td:last-child .objecttable-cellvalue"
            ).alltext_normalized,
            "Test image",
        )
        self.assertEqual(
            mockresponse.selector.one("#objecttableview-table tbody tr td:last-child .btn-default").alltext_normalized,
            "Use this",
        )

    def test_render_single_archive_image_has_description(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        archiveimage = baker.make(
            "cradmin_imagearchive.ArchiveImage", name="Test image", description="Test description", role=testrole
        )
        archiveimage.image.save("testimage.png", ContentFile(create_image(200, 100)))
        mockresponse = self.mock_http200_getrequest_htmls(
            requestuser=create_user("testuser"),
            cradmin_role=testrole,
            requestkwargs={"data": {"foreignkey_select_fieldid": "testfield"}},
        )
        self.assertEqual(
            mockresponse.selector.one(
                "#objecttableview-table tbody tr td:last-child .objecttable-cellvalue"
            ).alltext_normalized,
            "Test description",
        )
        self.assertEqual(
            mockresponse.selector.one("#objecttableview-table tbody tr td:last-child .btn-default").alltext_normalized,
            "Use this",
        )

    def test_uploadform_get_render(self):
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        mockresponse = self.mock_http200_getrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertTrue(mockresponse.selector.exists("#cradmin_legacy_imagearchive_bulkadd_form"))
        self.assertTrue(mockresponse.selector.exists("input[type=hidden][name=filecollectionid]"))
        self.assertTrue(mockresponse.selector.exists("#div_id_filecollectionid"))

    def test_uploadform_post_no_filecollectionid(self):
        self.assertEqual(ArchiveImage.objects.count(), 0)
        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        mockresponse = self.mock_http200_postrequest_htmls(requestuser=create_user("testuser"), cradmin_role=testrole)
        self.assertTrue(mockresponse.selector.exists("#div_id_filecollectionid.has-error"))
        self.assertIn(
            "You must upload an image.", mockresponse.selector.one("#div_id_filecollectionid").alltext_normalized
        )

    def test_uploadform_post_single_image(self):
        testuser = create_user("testuser")
        testimage = create_image(200, 100)
        collection = TemporaryFileCollection.objects.create(user=testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile.png")
        temporaryfile.file.save("testfile.png", ContentFile(testimage))

        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        cradmin_app = mock.MagicMock()
        self.assertEqual(ArchiveImage.objects.count(), 0)
        mockresponse = self.mock_http302_postrequest(
            requestuser=testuser,
            cradmin_role=testrole,
            cradmin_app=cradmin_app,
            requestkwargs={"data": {"filecollectionid": collection.id}},
        )

        self.assertEqual(ArchiveImage.objects.count(), 1)
        created_image = ArchiveImage.objects.first()
        self.assertTrue(
            mockresponse.response["Location"].endswith(f"?foreignkey_selected_value={created_image.pk}")
        )
        created_image.image.open()
        self.assertEqual(created_image.image.read(), testimage)
        created_image.image.close()
        self.assertEqual(created_image.name, "testfile.png")
        self.assertEqual(created_image.description, "")

    def test_uploadform_post_deletes_collection(self):
        testuser = create_user("testuser")
        testimage = create_image(200, 100)
        collection = TemporaryFileCollection.objects.create(user=testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile.png")
        temporaryfile.file.save("testfile.png", ContentFile(testimage))

        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        self.mock_http302_postrequest(
            requestuser=testuser, cradmin_role=testrole, requestkwargs={"data": {"filecollectionid": collection.id}}
        )

        self.assertFalse(TemporaryFileCollection.objects.filter(id=collection.id).exists())

    def test_uploadform_post_multiple_images_fails(self):
        testuser = create_user("testuser")
        testimage1 = create_image(200, 100)
        testimage2 = create_image(100, 100)
        collection = TemporaryFileCollection.objects.create(user=testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile1.png")
        temporaryfile.file.save("testfile1.png", ContentFile(testimage1))
        temporaryfile = TemporaryFile(collection=collection, filename="testfile2.png")
        temporaryfile.file.save("testfile2.png", ContentFile(testimage2))

        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        cradmin_app = mock.MagicMock()
        cradmin_app.reverse_appurl.return_value = "/success"

        self.assertEqual(ArchiveImage.objects.count(), 0)
        mockresponse = self.mock_http200_postrequest_htmls(
            requestuser=testuser,
            cradmin_role=testrole,
            cradmin_app=cradmin_app,
            requestkwargs={"data": {"filecollectionid": collection.id}},
        )
        self.assertEqual(ArchiveImage.objects.count(), 0)
        self.assertTrue(mockresponse.selector.exists("#div_id_filecollectionid.has-error"))
        self.assertIn(
            "You must upload exactly one image.",
            mockresponse.selector.one("#div_id_filecollectionid").alltext_normalized,
        )

    def test_uploadform_post_sets_file_size(self):
        testuser = create_user("testuser")
        testimage = create_image(200, 100)
        collection = TemporaryFileCollection.objects.create(user=testuser)
        temporaryfile = TemporaryFile(collection=collection, filename="testfile.png")
        temporaryfile.file.save("testfile.png", ContentFile(testimage))

        testrole = baker.make("cradmin_legacy_testapp.TstRole")
        self.mock_http302_postrequest(
            requestuser=testuser, cradmin_role=testrole, requestkwargs={"data": {"filecollectionid": collection.id}}
        )
        created_image = ArchiveImage.objects.first()
        self.assertEqual(len(testimage), created_image.file_size)



from crispy_forms import layout
from django.conf import settings
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy
from django import forms
from django.views.generic import FormView

from crispy_forms.helper import FormHelper

from cradmin_legacy.apps.cradmin_email import emailutils
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import (
    GenericTokenWithMetadata,
    get_expiration_datetime_for_app,
)
from cradmin_legacy.crispylayouts import PrimarySubmitLg


class PasswordResetEmail(emailutils.AbstractEmail):
    subject_template = "cradmin_resetpassword/email/subject.django.txt"
    html_message_template = "cradmin_resetpassword/email/html_message.django.html"

    def get_context_data(self):
        context = super().get_context_data()
        context.update({"CRADMIN_LEGACY_SITENAME": settings.CRADMIN_LEGACY_SITENAME})
        return context


class EmailForm(forms.Form):
    email = forms.EmailField()

    def clean_email(self):
        email = self.cleaned_data["email"]
        user_model = get_user_model()
        if not user_model.objects.filter(email=email).exists():
            raise forms.ValidationError(gettext_lazy("No account with this email address found"))
        return email


class BeginPasswordResetView(FormView):
    template_name = "cradmin_resetpassword/begin.django.html"
    form_class = EmailForm

    def get_formhelper(self):
        helper = FormHelper()
        helper.form_action = "#"
        helper.form_id = "cradmin_legacy_resetpassword_begin_form"
        helper.form_show_labels = False
        helper.layout = layout.Layout(
            layout.Field("email", css_class="input-lg", placeholder=gettext_lazy("Email"), focusonme="focusonme"),
            PrimarySubmitLg("submit", gettext_lazy("Search")),
        )
        return helper

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["formhelper"] = self.get_formhelper()
        return context

    def get_success_url(self):
        return str(reverse("cradmin-resetpassword-email-sent"))

    def __send_email(self, user, reset_url):
        PasswordResetEmail(
            recipient=user.email,
            from_email=getattr(settings, "CRADMIN_LEGACY_RESETPASSWORD_FROM_EMAIL", settings.DEFAULT_FROM_EMAIL),
            extra_context_data={"user": user, "reset_url": reset_url},
        ).send()

    def _generate_token(self, user):
        return GenericTokenWithMetadata.objects.generate(
            app="cradmin_resetpassword",
            content_object=user,
            expiration_datetime=get_expiration_datetime_for_app("cradmin_resetpassword"),
        ).token

    def __generate_reset_url(self, user):
        reset_url = reverse("cradmin-resetpassword-reset", kwargs={"token": self._generate_token(user)})
        return self.request.build_absolute_uri(reset_url)

    def get_user(self, email):
        user_model = get_user_model()
        return get_object_or_404(user_model, email=email)

    def form_valid(self, form):
        email = form.cleaned_data["email"]
        user = self.get_user(email)
        reset_url = self.__generate_reset_url(user=user)
        self.__send_email(user=user, reset_url=reset_url)
        return super().form_valid(form)



from urllib.parse import urlsplit

from django.http import QueryDict
from django.test import TestCase

from cradmin_legacy import urlutils


class TestUpdateQuerystring(TestCase):
    def test_simple(self):
        self.assertEqual(
            urlutils.update_querystring("http://example.com", {"next": "http://example.com/admin"}),
            "http://example.com?next=http%3A%2F%2Fexample.com%2Fadmin",
        )

    def test_update_existing(self):
        self.assertEqual(
            urlutils.update_querystring("http://example.com?next=/test", {"next": "/updated"}),
            "http://example.com?next=%2Fupdated",
        )

    def test_keep_existing(self):
        url = urlutils.update_querystring("http://example.com?next=/test", {"other": "otherstuff"})
        url_split = urlsplit(url)
        querydict = QueryDict(url_split.query)
        self.assertEqual(
            querydict.dict(),
            {
                "next": "/test",
                "other": "otherstuff",
            },
        )

    def test_ignore_none_values(self):
        self.assertEqual(urlutils.update_querystring("http://example.com", {"next": None}), "http://example.com")

    def test_do_not_ignore_none_values(self):
        self.assertEqual(
            urlutils.update_querystring("http://example.com", {"next": None}, ignore_none_values=False),
            "http://example.com?next=None",
        )



import htmls
from django import test
from django.core.files.base import ContentFile
from model_bakery import baker

from cradmin_legacy.apps.cradmin_imagearchive.tests.helpers import create_image
from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.viewhelpers import listbuilder


class TestTitleDescription(test.TestCase):
    def test_title(self):
        rendered = listbuilder.itemvalue.TitleDescription(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertEqual(
            "testvalue", selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-title").alltext_normalized
        )

    def test_without_description(self):
        rendered = listbuilder.itemvalue.TitleDescription(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertFalse(selector.exists(".cradmin-legacy-listbuilder-itemvalue-titledescription-description"))

    def test_with_description(self):
        class MyEditDelete(listbuilder.itemvalue.TitleDescription):
            def get_description(self):
                return "The description"

        rendered = MyEditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertEqual(
            "The description",
            selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-description").alltext_normalized,
        )


class MockValue:
    def __init__(self, value_str, pk):
        self.value_str = value_str
        self.pk = pk

    def __str__(self):
        return self.value_str


class TestUseThis(test.TestCase):
    def __make_mock_value(self, value_str, value_pk=1):
        # mockvalue = mock.MagicMock()
        # mockvalue.__str__.return_value = value_str
        # mockvalue.pk = value_pk
        # return mockvalue
        return MockValue(value_str=value_str, pk=value_pk)

    def __make_mock_request(self):
        mockrequest = mock.MagicMock()
        mockrequest.GET.__getitem__.return_value = "mocked"
        return mockrequest

    def test_title(self):
        rendered = listbuilder.itemvalue.UseThis(value=self.__make_mock_value("testvalue")).render(
            request=self.__make_mock_request()
        )
        selector = htmls.S(rendered)
        self.assertEqual(
            "testvalue", selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-title").alltext_normalized
        )

    def test_without_description(self):
        rendered = listbuilder.itemvalue.UseThis(value=self.__make_mock_value("testvalue")).render(
            request=self.__make_mock_request()
        )
        selector = htmls.S(rendered)
        self.assertFalse(selector.exists(".cradmin-legacy-listbuilder-itemvalue-titledescription-description"))

    def test_with_description(self):
        class MyEditDelete(listbuilder.itemvalue.UseThis):
            def get_description(self):
                return "The description"

        rendered = MyEditDelete(value=self.__make_mock_value("testvalue")).render(request=self.__make_mock_request())
        selector = htmls.S(rendered)
        self.assertEqual(
            "The description",
            selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-description").alltext_normalized,
        )

    def test_preview_label(self):
        rendered = listbuilder.itemvalue.UseThis(value=self.__make_mock_value("testvalue")).render(
            request=self.__make_mock_request()
        )
        selector = htmls.S(rendered)
        self.assertEqual(
            "Select", selector.one(".cradmin-legacy-listbuilder-itemvalue-usethis-usethis-button").alltext_normalized
        )

    def test_preview_aria_label(self):
        rendered = listbuilder.itemvalue.UseThis(value=self.__make_mock_value("testvalue")).render(
            request=self.__make_mock_request()
        )
        selector = htmls.S(rendered)
        self.assertEqual(
            'Select "testvalue"',
            selector.one(".cradmin-legacy-listbuilder-itemvalue-usethis-usethis-button")["aria-label"],
        )


class TestEditDelete(test.TestCase):
    def test_title(self):
        rendered = listbuilder.itemvalue.EditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertEqual(
            "testvalue", selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-title").alltext_normalized
        )

    def test_without_description(self):
        rendered = listbuilder.itemvalue.EditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertFalse(selector.exists(".cradmin-legacy-listbuilder-itemvalue-titledescription-description"))

    def test_with_description(self):
        class MyEditDelete(listbuilder.itemvalue.EditDelete):
            def get_description(self):
                return "The description"

        rendered = MyEditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertEqual(
            "The description",
            selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-description").alltext_normalized,
        )

    def test_edit_label(self):
        rendered = listbuilder.itemvalue.EditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertEqual(
            "Edit", selector.one(".cradmin-legacy-listbuilder-itemvalue-editdelete-editbutton").alltext_normalized
        )

    def test_edit_aria_label(self):
        rendered = listbuilder.itemvalue.EditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertEqual(
            'Edit "testvalue"',
            selector.one(".cradmin-legacy-listbuilder-itemvalue-editdelete-editbutton")["aria-label"],
        )

    def test_delete_label(self):
        rendered = listbuilder.itemvalue.EditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertEqual(
            "Delete", selector.one(".cradmin-legacy-listbuilder-itemvalue-editdelete-deletebutton").alltext_normalized
        )

    def test_delete_aria_label(self):
        rendered = listbuilder.itemvalue.EditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertEqual(
            'Delete "testvalue"',
            selector.one(".cradmin-legacy-listbuilder-itemvalue-editdelete-deletebutton")["aria-label"],
        )

    def test_viewbutton_not_rendered(self):
        rendered = listbuilder.itemvalue.EditDelete(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertFalse(selector.exists(".cradmin-legacy-listbuilder-itemvalue-editdelete-viewbutton"))


class TestEditDeleteWithPreview(test.TestCase):
    def test_preview_label(self):
        mockrequest = mock.MagicMock()
        mockrequest.cradmin_app.reverse_appurl.return_value = "/preview"
        rendered = listbuilder.itemvalue.EditDeleteWithPreview(value="testvalue").render(request=mockrequest)
        selector = htmls.S(rendered)
        self.assertEqual(
            "View", selector.one(".cradmin-legacy-listbuilder-itemvalue-editdelete-previewbutton").alltext_normalized
        )

    def test_preview_aria_label(self):
        mockrequest = mock.MagicMock()
        mockrequest.cradmin_app.reverse_appurl.return_value = "/preview"
        rendered = listbuilder.itemvalue.EditDeleteWithPreview(value="testvalue").render(request=mockrequest)
        selector = htmls.S(rendered)
        self.assertEqual(
            'View "testvalue"',
            selector.one(".cradmin-legacy-listbuilder-itemvalue-editdelete-previewbutton")["aria-label"],
        )


class TestEditDeleteWithArchiveImage(test.TestCase):
    def test_without_archiveimage(self):
        class MyEditDeleteWithArchiveImage(listbuilder.itemvalue.EditDeleteWithArchiveImage):
            def get_archiveimage(self):
                return None

        rendered = MyEditDeleteWithArchiveImage(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertFalse(selector.exists(".cradmin-legacy-listbuilder-itemvalue-editdelete-imagewrapper"))

    def test_with_archiveimage(self):
        class MyEditDeleteWithArchiveImage(listbuilder.itemvalue.EditDeleteWithArchiveImage):
            def get_archiveimage(self):
                archiveimage = baker.make("cradmin_imagearchive.ArchiveImage")
                archiveimage.image.save("testimage.png", ContentFile(create_image(200, 100)))
                return archiveimage

        mockrequest = mock.MagicMock()
        mockrequest.build_absolute_uri.return_value = "test"
        rendered = MyEditDeleteWithArchiveImage(value="testvalue").render(request=mockrequest)
        selector = htmls.S(rendered)
        self.assertTrue(selector.exists(".cradmin-legacy-listbuilder-itemvalue-editdelete-imagewrapper"))


class TestEditDeleteWithArchiveImageAndView(test.TestCase):
    def test_preview_label(self):
        class MyEditDeleteWithArchiveImageAndView(listbuilder.itemvalue.EditDeleteWithArchiveImageAndPreview):
            def get_archiveimage(self):
                return None

        mockrequest = mock.MagicMock()
        mockrequest.cradmin_app.reverse_appurl.return_value = "/preview"
        rendered = MyEditDeleteWithArchiveImageAndView(value="testvalue").render(request=mockrequest)
        selector = htmls.S(rendered)
        self.assertEqual(
            "View", selector.one(".cradmin-legacy-listbuilder-itemvalue-editdelete-previewbutton").alltext_normalized
        )

    def test_preview_aria_label(self):
        class MyEditDeleteWithArchiveImageAndView(listbuilder.itemvalue.EditDeleteWithArchiveImageAndPreview):
            def get_archiveimage(self):
                return None

        mockrequest = mock.MagicMock()
        mockrequest.cradmin_app.reverse_appurl.return_value = "/preview"
        rendered = MyEditDeleteWithArchiveImageAndView(value="testvalue").render(request=mockrequest)
        selector = htmls.S(rendered)
        self.assertEqual(
            'View "testvalue"',
            selector.one(".cradmin-legacy-listbuilder-itemvalue-editdelete-previewbutton")["aria-label"],
        )

    def test_without_archiveimage(self):
        class MyEditDeleteWithArchiveImageAndView(listbuilder.itemvalue.EditDeleteWithArchiveImageAndPreview):
            def get_archiveimage(self):
                return None

        rendered = MyEditDeleteWithArchiveImageAndView(value="testvalue").render(request=mock.MagicMock())
        selector = htmls.S(rendered)
        self.assertFalse(selector.exists(".cradmin-legacy-listbuilder-itemvalue-editdelete-imagewrapper"))

    def test_with_archiveimage(self):
        class MyEditDeleteWithArchiveImageAndView(listbuilder.itemvalue.EditDeleteWithArchiveImageAndPreview):
            def get_archiveimage(self):
                archiveimage = baker.make("cradmin_imagearchive.ArchiveImage")
                archiveimage.image.save("testimage.png", ContentFile(create_image(200, 100)))
                return archiveimage

        mockrequest = mock.MagicMock()
        mockrequest.build_absolute_uri.return_value = "test"
        rendered = MyEditDeleteWithArchiveImageAndView(value="testvalue").render(request=mockrequest)
        selector = htmls.S(rendered)
        self.assertTrue(selector.exists(".cradmin-legacy-listbuilder-itemvalue-editdelete-imagewrapper"))



import os
import posixpath
import uuid

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy

from cradmin_legacy import crsettings


def archiveimage_upload_to(archiveimage, filename):
    if archiveimage.id is None:
        raise ValueError("Can not set image until after the ArchiveImage object has been created.")
    name, extension = os.path.splitext(filename)
    filenamepattern = getattr(
        settings, "CRADMIN_LEGACY_IMAGEARCHIVE_FILENAMEPATTERN", "cradmin_imagearchive_images/{id}-{uuid}{extension}"
    )
    return filenamepattern.format(id=archiveimage.id, uuid=uuid.uuid1(), extension=extension)


class ArchiveImageManager(models.Manager):
    def filter_owned_by_role(self, role):
        return self.get_queryset().filter(
            role_object_id=role.id, role_content_type=ContentType.objects.get_for_model(role.__class__)
        )


class ArchiveImage(models.Model):
    objects = ArchiveImageManager()

    role_content_type = models.ForeignKey(
        ContentType,
        verbose_name=gettext_lazy("role"),
        help_text=gettext_lazy("The role owning this image."),
        on_delete=models.CASCADE,
    )
    role_object_id = models.PositiveIntegerField()
    role = GenericForeignKey("role_content_type", "role_object_id")

    #: The image.
    image = models.ImageField(
        max_length=255,
        null=True,
        blank=False,
        height_field="image_height",
        width_field="image_width",
        verbose_name=gettext_lazy("image"),
        help_text=gettext_lazy("Select an image to add to the archive."),
        upload_to=archiveimage_upload_to,
    )

    #: The height of the :obj:`.image`. Autopopulated by the :obj:`.image` field.
    image_height = models.IntegerField(null=True, blank=True, editable=False)

    #: The width of the :obj:`.image`. Autopopulated by the :obj:`.image` field.
    image_width = models.IntegerField(null=True, blank=True, editable=False)

    #: The file extension
    file_extension = models.CharField(max_length=255, blank=False, null=False, editable=False)

    #: The file size in bytes.
    file_size = models.PositiveIntegerField(blank=True, null=True)

    #: The name of the image file.
    #:
    #: Can be None. This is both because this was added
    #: after cradmin was in use in production, and because
    #: calculating size can be expensive if uploading
    #: huge amounts of images in bulk. All the cradmin views
    #: provided by the app sets this.
    name = models.CharField(
        max_length=255,
        blank=True,
        null=False,
        verbose_name=gettext_lazy("name"),
        # help_text=gettext_lazy('A good name helps search engines find the image, '
        #             'and it helps visually impaired users.')
    )

    #: An optional description of the image.
    description = models.TextField(
        blank=True,
        null=False,
        default="",
        verbose_name=gettext_lazy("description"),
        help_text=gettext_lazy(
            "An optional description of the image. Think if this as a description "
            "of the image for visually impaired users. This means that you should describe "
            "the information carried in the image (if any). A good description also helps "
            "search engines find the image."
        ),
    )

    #: Create datetime.
    created_datetime = models.DateTimeField(editable=False, auto_now_add=True)

    class Meta:
        verbose_name = gettext_lazy("archive image")
        verbose_name_plural = gettext_lazy("archive images")

    def clean(self):
        if not self.name:
            if self.image:
                self.name = os.path.splitext(posixpath.basename(self.image.name))[0]
        if self.name:
            self.file_extension = os.path.splitext(self.name)[1]

    def __str__(self):
        return self.name

    def __repr__(self):
        return "ArchiveImage(name={name}, image_width={image_width}, image_height={image_height})".format(
            name=self.name.encode("ascii", "replace"), image_width=self.image_width, image_height=self.image_height
        )

    def get_preview_html(self, request, imagetype=None):
        """
        Get HTML that can be used to show a preview of the image.

        Parameters:
            request: A HttpRequest object which is used to create an
                absolute URI for the image.
        """
        if not imagetype:
            imagetype = crsettings.get_setting("CRADMIN_LEGACY_IMAGEARCHIVE_PREVIEW_IMAGETYPE")
        context = {"archiveimage": self, "imagetype": imagetype, "fallbackoptions": {"width": 300, "height": 300}}
        return render_to_string(
            "cradmin_legacy/apps/cradmin_imagearchive/preview.django.html", context, request=request
        )

    @property
    def screenreader_text(self):
        """
        Get screen reader text for the image. Typically added to the "alt"
        or "aria-label" HTML attributes.

        Defaults to description, but falls back to name.
        """
        if self.description:
            return self.description
        else:
            return self.name



import json
import mimetypes
from django.conf import settings
from django.core.exceptions import ValidationError
from django.http import HttpResponse
from django.views.generic import FormView
from django.utils.translation import gettext_lazy

from django import forms
from multiupload.fields import MultiFileField
from cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models import (
    TemporaryFileCollection,
    TemporaryFile,
    html_input_accept_match,
    make_unique_filename,
    validate_max_file_size,
)


class FileUploadForm(forms.Form):
    file = MultiFileField(
        max_file_size=1000000 * getattr(settings, "CRADMIN_TEMPORARYFILEUPLOADSTORE_MAX_FILE_SIZE_MB", 1000), min_num=1
    )
    collectionid = forms.IntegerField(required=False)
    minutes_to_live = forms.IntegerField(min_value=1, required=False)
    singlemode = forms.BooleanField(required=False)
    accept = forms.CharField(required=False)
    max_filename_length = forms.IntegerField(min_value=0, required=False)
    max_filesize_bytes = forms.IntegerField(min_value=0, required=False)
    unique_filenames = forms.BooleanField(required=False)


class FileDeleteForm(forms.Form):
    collectionid = forms.IntegerField(required=True)
    temporaryfileid = forms.IntegerField(required=True)


class UploadTemporaryFilesView(FormView):
    form_class = FileUploadForm
    http_method_names = ["post", "delete"]

    def create_collection(
        self, minutes_to_live, accept, max_filename_length, max_filesize_bytes, unique_filenames, singlemode
    ):
        collection = TemporaryFileCollection(user=self.request.user)
        if minutes_to_live is not None:
            collection.minutes_to_live = minutes_to_live
        if accept is not None:
            collection.accept = accept
        if max_filename_length:
            collection.max_filename_length = max_filename_length
        if unique_filenames:
            collection.unique_filenames = unique_filenames
        if singlemode:
            collection.singlemode = singlemode
        if max_filesize_bytes:
            collection.max_filesize_bytes = max_filesize_bytes
        collection.full_clean()
        collection.save()
        return collection

    def get_existing_collection(self, collectionid):
        return TemporaryFileCollection.objects.filter(user=self.request.user).get(id=collectionid)

    def create_or_get_collection_id(
        self,
        collectionid,
        minutes_to_live,
        accept,
        max_filename_length,
        max_filesize_bytes,
        unique_filenames,
        singlemode,
    ):
        if collectionid is None:
            return self.create_collection(
                minutes_to_live=minutes_to_live,
                accept=accept,
                max_filename_length=max_filename_length,
                max_filesize_bytes=max_filesize_bytes,
                unique_filenames=unique_filenames,
                singlemode=singlemode,
            )
        else:
            return self.get_existing_collection(collectionid)

    def save_uploaded_file(self, collection, formfile, filename_set):
        filename = formfile.name
        if collection.unique_filenames:
            filename = make_unique_filename(
                filename_set=filename_set, wanted_filename=filename, max_filename_length=collection.max_filename_length
            )
            filename_set.add(filename)
        temporaryfile = TemporaryFile(collection=collection, filename=filename)
        temporaryfile.file.save(formfile.name, formfile, save=False)
        temporaryfile.full_clean()
        temporaryfile.save()
        return temporaryfile

    def form_invalid(self, form):
        return self.__invalid_form_response(form)

    def everything_valid(self, collection, form):
        uploadedfiles_data = []
        filename_set = None
        if collection.unique_filenames:
            filename_set = collection.get_filename_set()
        for formfile in form.cleaned_data["file"]:
            temporaryfile = self.save_uploaded_file(collection=collection, formfile=formfile, filename_set=filename_set)
            uploadedfiles_data.append(
                {"id": temporaryfile.id, "filename": temporaryfile.filename, "mimetype": temporaryfile.mimetype}
            )
        return self.json_response(json.dumps({"collectionid": collection.id, "temporaryfiles": uploadedfiles_data}))

    def __validate_accept(self, accept, form):
        if not accept:
            return
        for formfile in form.cleaned_data["file"]:
            if not html_input_accept_match(
                accept=accept, mimetype=mimetypes.guess_type(formfile.name)[0], filename=formfile.name
            ):
                raise ValidationError(
                    gettext_lazy("%(filename)s: Unsupported filetype.") % {"filename": formfile.name},
                    code="unsupported_mimetype",
                )

    def __validate_max_filesize_bytes(self, form, max_filesize_bytes):
        if max_filesize_bytes is None:
            return
        for formfile in form.cleaned_data["file"]:
            validate_max_file_size(max_filesize_bytes=max_filesize_bytes, fieldfile=formfile)

    def validate_all_files(self, accept, form, max_filesize_bytes):
        self.__validate_accept(accept=accept, form=form)
        self.__validate_max_filesize_bytes(form=form, max_filesize_bytes=max_filesize_bytes)

    def form_valid(self, form):
        collectionid = form.cleaned_data["collectionid"]
        minutes_to_live = form.cleaned_data["minutes_to_live"]
        accept = form.cleaned_data["accept"]
        max_filename_length = form.cleaned_data["max_filename_length"]
        max_filesize_bytes = form.cleaned_data["max_filesize_bytes"]
        unique_filenames = form.cleaned_data["unique_filenames"]
        singlemode = form.cleaned_data["singlemode"]
        try:
            # NOTE: We validate all before saving any data to the models to avoid
            #       creating files in the storage backend before it is needed.
            self.validate_all_files(accept=accept, form=form, max_filesize_bytes=max_filesize_bytes)
        except ValidationError as e:
            return self.json_response(json.dumps({"file": [{"message": e.message, "code": e.code}]}), status=400)
        else:
            try:
                collection = self.create_or_get_collection_id(
                    collectionid=collectionid,
                    minutes_to_live=minutes_to_live,
                    accept=accept,
                    max_filename_length=max_filename_length,
                    max_filesize_bytes=max_filesize_bytes,
                    unique_filenames=unique_filenames,
                    singlemode=singlemode,
                )
            except TemporaryFileCollection.DoesNotExist:
                return self.__collection_does_not_exist_response(collectionid)
            else:
                return self.everything_valid(collection=collection, form=form)

    def json_response(self, data, status=200):
        return HttpResponse(data, content_type="application/json", status=status)

    def __collection_does_not_exist_response(self, collectionid):
        return self.json_response(
            json.dumps(
                {
                    "collectionid": [
                        {
                            "message": f"Collection with id={collectionid} does not exist.",
                            "code": "doesnotexist",
                        }
                    ]
                }
            ),
            status=404,
        )

    def __temporaryfile_does_not_exist_response(self, temporaryfileid):
        return self.json_response(
            json.dumps(
                {
                    "temporaryfileid": [
                        {
                            "message": f"Temporary file with id={temporaryfileid} does not exist.",
                            "code": "doesnotexist",
                        }
                    ]
                }
            ),
            status=404,
        )

    def __invalid_form_response(self, form):
        return self.json_response(form.errors.as_json(), status=400)

    def delete(self, request, *args, **kwargs):
        try:
            requestdata = json.loads(self.request.body.decode("utf-8"))
        except ValueError:
            return self.json_response(
                json.dumps({"errormessage": "Invalid JSON data in the request body."}), status=400
            )

        form = FileDeleteForm(requestdata)
        if form.is_valid():
            collectionid = form.cleaned_data["collectionid"]
            temporaryfileid = form.cleaned_data["temporaryfileid"]
            try:
                collection = self.get_existing_collection(collectionid)
            except TemporaryFileCollection.DoesNotExist:
                return self.__collection_does_not_exist_response(collectionid)
            else:
                try:
                    temporaryfile = collection.files.get(id=temporaryfileid)
                except TemporaryFile.DoesNotExist:
                    return self.__temporaryfile_does_not_exist_response(temporaryfileid)
                else:
                    return self.perform_temporaryfile_delete(temporaryfile)
        else:
            return self.__invalid_form_response(form)

    def perform_temporaryfile_delete(self, temporaryfile):
        temporaryfileid = temporaryfile.id
        temporaryfile.delete_object_and_file()
        return self.json_response(
            json.dumps({"collectionid": temporaryfile.collection_id, "temporaryfileid": temporaryfileid})
        )



from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.test import TestCase
from cradmin_legacy.apps.cradmin_register_account.forms.auth_user import (
    AuthUserCreateAccountForm,
    AuthUserCreateAccountAutoUsernameForm,
)
from cradmin_legacy.tests.helpers import create_user


class TestAuthUserCreateAccountForm(TestCase):
    def test_is_valid(self):
        form = AuthUserCreateAccountForm(
            {"username": "test", "password1": "test", "password2": "test", "email": "test@example.com"}
        )
        self.assertTrue(form.is_valid())

    def test_save(self):
        form = AuthUserCreateAccountForm(
            {"username": "test", "password1": "test", "password2": "test", "email": "test@example.com"}
        )
        form.is_valid()
        self.assertEqual(User.objects.count(), 0)
        form.save()
        self.assertEqual(User.objects.count(), 1)
        created_user = User.objects.first()
        self.assertEqual(created_user.username, "test")
        self.assertEqual(created_user.email, "test@example.com")
        self.assertTrue(created_user.has_usable_password())
        self.assertTrue(created_user.check_password("test"))

    def test_passwords_do_not_match(self):
        form = AuthUserCreateAccountForm(
            {"username": "unused", "password1": "test1", "password2": "test2", "email": "unused@example.com"}
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(form.non_field_errors(), ["The passwords do not match."])

    def test_email_is_not_unique(self):
        create_user("testuser", email="test@example.com")
        form = AuthUserCreateAccountForm(
            {"username": "unused", "password1": "unused", "password2": "unused", "email": "test@example.com"}
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors, {"email": ["Account with this email address already exists."]})

    def test_username_is_not_unique(self):
        create_user("test")
        form = AuthUserCreateAccountForm(
            {"username": "test", "password1": "unused", "password2": "unused", "email": "unused@example.com"}
        )
        self.assertFalse(form.is_valid())
        self.assertIn("username", form.errors)
        self.assertIn("name already exists", form.errors["username"][0])


class TestAuthUserCreateAccountAutoUsernameForm(TestCase):
    def test_is_valid(self):
        form = AuthUserCreateAccountAutoUsernameForm(
            {"password1": "test", "password2": "test", "email": "test@example.com"}
        )
        self.assertTrue(form.is_valid())

    def test_save(self):
        form = AuthUserCreateAccountAutoUsernameForm(
            {"password1": "test", "password2": "test", "email": "test@example.com"}
        )
        form.is_valid()
        self.assertEqual(User.objects.count(), 0)
        form.save()
        self.assertEqual(User.objects.count(), 1)
        created_user = User.objects.first()
        self.assertEqual(created_user.username, "test@example.com")
        self.assertEqual(created_user.email, "test@example.com")
        self.assertTrue(created_user.has_usable_password())
        self.assertTrue(created_user.check_password("test"))

    def test_passwords_do_not_match(self):
        form = AuthUserCreateAccountAutoUsernameForm(
            {"password1": "test1", "password2": "test2", "email": "unused@example.com"}
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(form.non_field_errors(), ["The passwords do not match."])

    def test_email_is_not_unique(self):
        create_user("testuser", email="test@example.com")
        form = AuthUserCreateAccountAutoUsernameForm(
            {"password1": "unused", "password2": "unused", "email": "test@example.com"}
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors, {"email": ["Account with this email address already exists."]})

    def test_email_is_more_than_30_chars(self):
        create_user("testuser", email="test@example.com")
        form = AuthUserCreateAccountAutoUsernameForm(
            {"password1": "unused", "password2": "unused", "email": "a.very.long.testuser@example.com"}
        )
        self.assertTrue(form.is_valid())
        user = form.save()
        self.assertEqual(user.username, "a.very.long.testuser@example.c")

    def test_email_makes_username_not_unique(self):
        create_user(username="a.very.long.testuser@example.c", email="a.very.long.testuser@example.co.uk")
        form = AuthUserCreateAccountAutoUsernameForm(
            {"password1": "unused", "password2": "unused", "email": "a.very.long.testuser@example.com"}
        )
        self.assertTrue(form.is_valid())
        with self.assertRaisesRegex(ValidationError, "^.*name already exists.*$"):
            form.save()



import htmls
from django import test
from django.http import QueryDict

from cradmin_legacy import python2_compatibility
from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.viewhelpers.multiselect2 import manytomanywidget


class TestWidget(test.TestCase):
    def test_wrapper_div_id(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertTrue(selector.exists("#id_cradmin_modelmultichoicefieldwrapper_testname"))

    def test_wrapper_class(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertTrue(
            selector.one("#id_cradmin_modelmultichoicefieldwrapper_testname").hasclass(
                "cradmin-legacy-modelmultichoicefield-wrapper"
            )
        )

    def test_wrapper_angularjs_directive_value(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=[1, 2], attrs={"id": "testid"}
            )
        )
        directive_value = selector.one("#id_cradmin_modelmultichoicefieldwrapper_testname")[
            "cradmin-legacy-model-choice-field-wrapper"
        ]
        path, querydictstring = directive_value.split("?")
        querydict = QueryDict(querydictstring)
        self.assertEqual("/test", path)
        self.assertEqual(
            {
                "manytomany_select_fieldid": "testid",
                "manytomany_select_current_value": "[1, 2]",
                "manytomany_select_required": "True",
            },
            querydict.dict(),
        )

    def test_inputfield_value_none(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertEqual("[]", selector.one('input[name="testname"]')["value"])

    def test_inputfield_value_empty_string(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value="", attrs={"id": "testid"}
            )
        )
        self.assertEqual("[]", selector.one('input[name="testname"]')["value"])

    def test_inputfield_value(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=[1, 2], attrs={"id": "testid"}
            )
        )
        self.assertEqual("[1, 2]", selector.one('input[name="testname"]')["value"])

    def test_inputfield_type_input_field_visible_default(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertEqual("hidden", selector.one('input[name="testname"]')["type"])

    def test_inputfield_type_input_field_visible_false(self):
        class MyWidget(manytomanywidget.Widget):
            input_field_visible = False

        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            MyWidget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertEqual("hidden", selector.one('input[name="testname"]')["type"])

    def test_inputfield_type_input_field_visible_true(self):
        class MyWidget(manytomanywidget.Widget):
            input_field_visible = True

        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            MyWidget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertEqual("text", selector.one('input[name="testname"]')["type"])

    def test_preview_list_sanity(self):
        mockqueryset = mock.MagicMock()
        mockqueryset.filter.return_value = []
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertTrue(selector.exists(".cradmin-legacy-multiselect2-preview-list"))

    def test_preview_list_angularjs_directive(self):
        mockqueryset = mock.MagicMock()
        mockqueryset.filter.return_value = []
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertTrue(
            selector.one(".cradmin-legacy-multiselect2-preview-list").hasattribute(
                "cradmin-legacy-model-choice-field-preview"
            )
        )

    def test_preview_list_without_value(self):
        mockqueryset = mock.MagicMock()
        mockqueryset.filter.return_value = []
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertEqual(0, selector.count(".cradmin-legacy-multiselect2-preview-list li"))

    def test_preview_list_with_value(self):
        mockqueryset = mock.MagicMock()
        mockmodelobject = mock.MagicMock()
        mockmodelobject.pk = 1
        mockmodelobject.__str__.return_value = "testvalue"
        if python2_compatibility.is_python2():
            mockmodelobject.__unicode__.return_value = "testvalue"
        mockqueryset.filter.return_value = [mockmodelobject]
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertEqual(1, selector.count(".cradmin-legacy-multiselect2-preview-list li"))
        self.assertEqual(
            "testvalue",
            selector.one(
                ".cradmin-legacy-multiselect2-preview-list .cradmin-legacy-multiselect2-preview-list-value"
            ).alltext_normalized,
        )

    def test_selectbutton_angularjs_directive(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertTrue(selector.one("button").hasattribute("cradmin-legacy-model-choice-field-changebegin-button"))

    def test_selectbutton_css_class(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertEqual("btn btn-default", selector.one("button")["class"])

    def test_selectbutton_text_default(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(queryset=mockqueryset, selectview_url="/test").render(
                name="testname", value=None, attrs={"id": "testid"}
            )
        )
        self.assertEqual("Select ...", selector.one("button").alltext_normalized)

    def test_selectbutton_text_custom(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(
            manytomanywidget.Widget(
                queryset=mockqueryset, selectview_url="/test", selectbutton_text="Testing ..."
            ).render(name="testname", value=None, attrs={"id": "testid"})
        )
        self.assertEqual("Testing ...", selector.one("button").alltext_normalized)



import json

import htmls
from django import test

from cradmin_legacy import python2_compatibility
from cradmin_legacy.viewhelpers.multiselect2 import listbuilder_itemvalues
from cradmin_legacy.python2_compatibility import mock


class TestItemValue(test.TestCase):
    def test_title(self):
        mockvalue = mock.MagicMock()
        mockvalue.__str__.return_value = "testvalue"
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = "testvalue"

        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue).render())
        self.assertEqual(
            "testvalue", selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-title").alltext_normalized
        )

    def test_no_description(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue).render())
        self.assertFalse(selector.exists(".cradmin-legacy-listbuilder-itemvalue-titledescription-description"))

    def test_has_description(self):
        class MySelectedItem(listbuilder_itemvalues.ItemValue):
            def get_description(self):
                return "test description"

        mockvalue = mock.MagicMock()
        selector = htmls.S(MySelectedItem(value=mockvalue).render())
        self.assertEqual(
            "test description",
            selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-description").alltext_normalized,
        )

    def test_selectbutton_text(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue).render())
        self.assertEqual(
            "Select", selector.one("button.cradmin-legacy-multiselect2-itemvalue-button").alltext_normalized
        )

    def test_selectbutton_aria_label(self):
        mockvalue = mock.MagicMock()
        mockvalue.__str__.return_value = "testvalue"
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = "testvalue"

        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue).render())
        self.assertEqual(
            'Select "testvalue"', selector.one("button.cradmin-legacy-multiselect2-itemvalue-button")["aria-label"]
        )

    def test_selectbutton_id(self):
        mockvalue = mock.MagicMock()
        mockvalue.pk = 10
        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue).render())
        self.assertTrue(selector.exists("button#cradmin_legacy_multiselect2_selectbutton_10"))

    def test_angularjs_directive(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue).render())
        directiveoptions = json.loads(
            selector.one("button.cradmin-legacy-multiselect2-itemvalue-button")["cradmin-legacy-multiselect2-select"]
        )
        self.assertEqual(
            {
                "custom_data": None,
                "is_selected": False,
                "item_wrapper_css_selector": "li",
                "preview_container_css_selector": ".cradmin-legacy-multiselect2-itemvalue",
                "preview_css_selector": ".cradmin-legacy-multiselect2-selected-item",
                "target_dom_id": "cradmin_legacy_multiselect2_select_target",
            },
            directiveoptions,
        )

    def test_angularjs_directive_custom_target_dom_id(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue, target_dom_id="testid").render())
        directiveoptions = json.loads(
            selector.one("button.cradmin-legacy-multiselect2-itemvalue-button")["cradmin-legacy-multiselect2-select"]
        )
        self.assertEqual("testid", directiveoptions["target_dom_id"])

    def test_angularjs_directive_is_selected_false(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue, is_selected=False).render())
        directiveoptions = json.loads(
            selector.one("button.cradmin-legacy-multiselect2-itemvalue-button")["cradmin-legacy-multiselect2-select"]
        )
        self.assertFalse(directiveoptions["is_selected"])

    def test_angularjs_directive_is_selected_true(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(value=mockvalue, is_selected=True).render())
        directiveoptions = json.loads(
            selector.one("button.cradmin-legacy-multiselect2-itemvalue-button")["cradmin-legacy-multiselect2-select"]
        )
        self.assertTrue(directiveoptions["is_selected"])


class TestManyToManySelect(test.TestCase):
    def test_sanity(self):
        mockvalue = mock.MagicMock()
        mockvalue.__str__.return_value = "testvalue"
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = "testvalue"
        mockvalue.pk = "unused"
        selector = htmls.S(listbuilder_itemvalues.ManyToManySelect(value=mockvalue).render())
        self.assertEqual(
            "testvalue", selector.one(".cradmin-legacy-listbuilder-itemvalue-titledescription-title").alltext_normalized
        )

    def test_angularjs_directive_custom_data_value(self):
        mockvalue = mock.MagicMock()
        mockvalue.pk = 21
        selector = htmls.S(listbuilder_itemvalues.ManyToManySelect(value=mockvalue, target_dom_id="testid").render())
        directiveoptions = json.loads(
            selector.one("button.cradmin-legacy-multiselect2-itemvalue-button")["cradmin-legacy-multiselect2-select"]
        )
        self.assertEqual(21, directiveoptions["custom_data"]["value"])

    def test_angularjs_directive_custom_data_preview(self):
        mockvalue = mock.MagicMock()
        mockvalue.pk = "unused"
        mockvalue.__str__.return_value = "testvalue"
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = "testvalue"
        selector = htmls.S(listbuilder_itemvalues.ManyToManySelect(value=mockvalue, target_dom_id="testid").render())
        directiveoptions = json.loads(
            selector.one("button.cradmin-legacy-multiselect2-itemvalue-button")["cradmin-legacy-multiselect2-select"]
        )
        selector = htmls.S(directiveoptions["custom_data"]["preview"])
        self.assertEqual(
            "testvalue", selector.one(".cradmin-legacy-multiselect2-preview-list-value").alltext_normalized
        )



from django.core import mail

from django.test import TestCase
from cradmin_legacy.python2_compatibility import mock

from cradmin_legacy.apps.cradmin_activate_account.utils import ActivationEmail
from cradmin_legacy.tests.helpers import create_user


class TestSendActivationEmail(TestCase):
    def setUp(self):
        self.testuser = create_user("testuser", email="testuser@example.com")

    def test_generate_token(self):
        token = ActivationEmail(user=self.testuser, next_url="/", request=mock.MagicMock()).generate_token()
        self.assertEqual(token.metadata["next_url"], "/")

    def test_generate_token_no_next_url(self):
        with self.settings(LOGIN_URL="/next"):
            token = ActivationEmail(user=self.testuser, request=mock.MagicMock()).generate_token()
        self.assertEqual(token.metadata["next_url"], "/next")

    def test_send(self):
        testtoken = mock.MagicMock()
        testtoken.token = "test-token"
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: f"http://testserver{path}"

        with mock.patch.object(ActivationEmail, "generate_token", lambda s: testtoken):
            with self.settings(CRADMIN_LEGACY_SITENAME="Testsite"):
                ActivationEmail(user=self.testuser, request=testrequest).send()
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "Activate your Testsite account")
        self.assertIn(
            "http://testserver/cradmin_activate_account/activate/test-token", mail.outbox[0].alternatives[0][0]
        )
        self.assertIn(
            "Click the button below to activate your Testsite account, testuser.", mail.outbox[0].alternatives[0][0]
        )



from django.core.management.base import BaseCommand
from cradmin_legacy.apps.cradmin_email import emailutils


class Command(BaseCommand):
    def handle(self, args, **kwargs):
        class DemoEmail(emailutils.AbstractEmail):
            subject_template = "cradmin_email/cradmin_email_send_testmail/subject.django.txt"
            html_message_template = "cradmin_email/cradmin_email_send_testmail/html_message.django.html"

        DemoEmail(recipient="post@appresso.no", extra_context_data={"name": "Test Name"}).send()



from django.forms import widgets


class WrappedSelect(widgets.Select):
    def __init__(self, *args, **kwargs):
        self.wrapper_css_class = kwargs.pop("wrapper_css_class", "")
        super().__init__(*args, **kwargs)

    def render(self, *args, **kwargs):
        rendered_widget = super().render(*args, **kwargs)
        return '<div class="{}">{}</div>'.format(self.wrapper_css_class, "".join(rendered_widget))



from model_bakery import baker

from cradmin_legacy.python2_compatibility import mock
import htmls
from django.test import TestCase
from django.test.client import RequestFactory
from django import http
from django import forms

from cradmin_legacy.tests.viewhelpers.cradmin_viewhelpers_testapp.models import TestModel
from cradmin_legacy.viewhelpers import multiselect

TEST_PK = 42


class DemoForm(forms.Form):
    data = forms.CharField()


class SimpleMultiSelectFormView(multiselect.MultiSelectFormView):
    form_class = DemoForm
    model = TestModel

    def get_queryset_for_role(self, role):
        return TestModel.objects.all()

    def form_valid(self, form):
        return http.HttpResponse("Submitted: {data}".format(**form.cleaned_data))

    def get_field_layout(self):
        return ["data"]


class SimpleMultiSelectView(multiselect.MultiSelectView):
    def get_queryset_for_role(self, role):
        return TestModel.objects.all()

    def object_selection_valid(self):
        return http.HttpResponse("OK")


class TestMultiSelectView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.testmodelinstance = baker.make("cradmin_viewhelpers_testapp.TestModel", pk=TEST_PK)

    def test_object_selection_valid(self):
        testmodelinstance2 = baker.make("cradmin_viewhelpers_testapp.TestModel")
        testmodelinstance3 = baker.make("cradmin_viewhelpers_testapp.TestModel")

        request = self.factory.post("/test", {"selected_objects": [testmodelinstance2.id, testmodelinstance3.id]})
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"OK")

    def test_object_selection_invalid(self):
        request = self.factory.post("/test", {"selected_objects": [1, 10]})
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectView.as_view()(request)
        selector = htmls.S(response.content)
        self.assertEqual(
            selector.one(".alert.alert-danger").text_normalized,
            "Invalid selection. This is usually caused by someone else changing permissions "
            "while you where selecting items to edit.",
        )

    def test_object_selection_invalid_override(self):
        class SimpleMultiSelectViewOverride(SimpleMultiSelectView):
            def object_selection_invalid(self, form):
                return http.HttpResponse("Invalid selection")

        request = self.factory.post("/test", {"selected_objects": [1, 10]})
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectViewOverride.as_view()(request)
        self.assertIn(b"Invalid selection", response.content)


class TestMultiSelectFormView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.testmodelinstance = baker.make("cradmin_viewhelpers_testapp.TestModel", pk=TEST_PK)

    def test_first_load(self):

        request = self.factory.post(
            "/test", {"selected_objects": [self.testmodelinstance.id], "is_the_multiselect_form": "yes"}
        )
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count("#cradmin_legacy_contentwrapper form"), 1)
        self.assertEqual(selector.one("input[name=selected_objects]")["type"], "hidden")
        self.assertEqual(selector.one("input[name=selected_objects]")["value"], f"{self.testmodelinstance.id}")
        self.assertEqual(selector.count("input[name=data]"), 1)
        self.assertFalse(selector.exists("form .has-error"))

    def test_form_invalid(self):

        request = self.factory.post(
            "/test",
            {
                "selected_objects": [self.testmodelinstance.id],
            },
        )
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count("#cradmin_legacy_contentwrapper form"), 1)
        self.assertEqual(selector.one("input[name=selected_objects]")["type"], "hidden")
        self.assertEqual(selector.one("input[name=selected_objects]")["value"], f"{self.testmodelinstance.id}")
        self.assertEqual(selector.count("input[name=data]"), 1)
        self.assertEqual(selector.one("#div_id_data .help-block").alltext_normalized, "This field is required.")

    def test_form_valid(self):

        request = self.factory.post("/test", {"selected_objects": [self.testmodelinstance.id], "data": "Hello world"})
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        self.assertEqual(response.content, b"Submitted: Hello world")

    def test_form_valid_success_redirect(self):
        class DemoForm(forms.Form):
            data = forms.CharField()

        class SimpleMultiSelectFormView(multiselect.MultiSelectFormView):
            form_class = DemoForm
            model = TestModel

            def get_queryset_for_role(self, role):
                return TestModel.objects.filter(id=TEST_PK)

            def get_success_url(self):
                return "/success"

            def form_valid(self, form):
                return self.success_redirect_response()

        request = self.factory.post("/test", {"selected_objects": [self.testmodelinstance.id], "data": "Hello world"})
        request.cradmin_role = mock.MagicMock()
        response = SimpleMultiSelectFormView.as_view()(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response["Location"], "/success")



#: Font-awesome icon map for the ``cradmin_icon`` template tag.
FONT_AWESOME = {
    "search": "fa fa-search",
    "x": "fa fa-times",
    "h1": "fa fa-header cradmin-legacy-icon-font-lg",
    "h2": "fa fa-header",
    "h3": "fa fa-header cradmin-legacy-icon-font-sm",
    "bold": "fa fa-bold",
    "italic": "fa fa-italic",
    "link": "fa fa-link",
    "list-ul": "fa fa-list-ul",
    "list-ol": "fa fa-list-ol",
    "codeblock": "fa fa-code",
    "loadspinner": "fa fa-spin fa-spinner",
    "caret-down": "fa fa-caret-down",
    "caret-up": "fa fa-caret-up",
    "pager-next-page": "fa fa-chevron-right",
    "pager-previus-page": "fa fa-chevron-left",
    "close-overlay-right-to-left": "fa fa-chevron-left",
    "select-left": "fa fa-angle-right",
    "select-right": "fa fa-angle-left",
    "multiselect2-selected": "fa fa-check",
}



from django.utils.html import format_html
import htmls
from cradmin_legacy.python2_compatibility import mock
from django.test import TestCase, RequestFactory
from mock_django.query import QuerySetMock

from cradmin_legacy.views.roleselect import RoleSelectView


class TestRoleSelectView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_302_redirect_on_single_role(self):
        cradmin_instance = mock.MagicMock()
        cradmin_instance.get_rolequeryset.return_value.count = mock.MagicMock(return_value=1)
        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch("cradmin_legacy.views.roleselect.cradmin_instance_registry", cradmin_instance_registry):
            request = self.factory.get("/roleselecttest")
            response = RoleSelectView.as_view()(request)
            self.assertEqual(response.status_code, 302)

    def test_200_on_multiple(self):
        cradmin_instance = mock.MagicMock()
        cradmin_instance.get_rolequeryset.return_value.count = mock.MagicMock(return_value=2)
        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch("cradmin_legacy.views.roleselect.cradmin_instance_registry", cradmin_instance_registry):
            request = self.factory.get("/roleselecttest")
            response = RoleSelectView.as_view()(request)
            self.assertEqual(response.status_code, 200)

    def __mock_cradmin_instance(self, roles):
        cradmin_instance = mock.MagicMock()
        cradmin_instance.get_rolequeryset.return_value.count = mock.MagicMock(return_value=len(roles))
        cradmin_instance.get_rolequeryset.return_value.__len__.return_value = len(roles)
        cradmin_instance.get_titletext_for_role = lambda role: role.title
        cradmin_instance.get_descriptionhtml_for_role = lambda role: format_html("<p>{}<p>", role.description)
        cradmin_instance.get_roleid = lambda role: role.id
        cradmin_instance.rolefrontpage_url = lambda roleid: f"/role/{roleid}"
        cradmin_instance.get_rolequeryset.return_value = QuerySetMock(None, *roles)
        return cradmin_instance

    def test_render_pagetitle(self):
        class CustomRoleSelectView(RoleSelectView):
            pagetitle = "Test title"

        role1 = mock.MagicMock()
        role1.id = 1
        role1.title = "Role One"
        role2 = mock.MagicMock()
        role2.id = 2
        role2.title = "Role Two"
        cradmin_instance = self.__mock_cradmin_instance(roles=[role1, role2])
        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch("cradmin_legacy.views.roleselect.cradmin_instance_registry", cradmin_instance_registry):
            with mock.patch(
                "cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry", cradmin_instance_registry
            ):
                request = self.factory.get("/roleselecttest")
                response = CustomRoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)
                self.assertEqual(selector.one("h1").alltext_normalized, "Test title")

    def test_render_list_titles(self):
        role1 = mock.MagicMock()
        role1.id = 1
        role1.title = "Role One"
        role2 = mock.MagicMock()
        role2.id = 2
        role2.title = "Role Two"
        cradmin_instance = self.__mock_cradmin_instance(roles=[role1, role2])

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch("cradmin_legacy.views.roleselect.cradmin_instance_registry", cradmin_instance_registry):
            with mock.patch(
                "cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry", cradmin_instance_registry
            ):
                request = self.factory.get("/roleselecttest")
                response = RoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)

                self.assertEqual(selector.count(".cradmin-legacy-roleselect-role-title"), 2)
                titletextlist = [
                    element.alltext_normalized for element in selector.list(".cradmin-legacy-roleselect-role-title")
                ]
                self.assertEqual(titletextlist, ["Role One", "Role Two"])

    def test_render_list_descriptions(self):
        role1 = mock.MagicMock()
        role1.id = 1
        role1.title = "Role One"
        role1.description = "Role One desc"
        role2 = mock.MagicMock()
        role2.id = 2
        role2.title = "Role Two"
        role2.description = "Role Two desc"
        cradmin_instance = self.__mock_cradmin_instance(roles=[role1, role2])

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch("cradmin_legacy.views.roleselect.cradmin_instance_registry", cradmin_instance_registry):
            with mock.patch(
                "cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry", cradmin_instance_registry
            ):
                request = self.factory.get("/roleselecttest")
                response = RoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)

                self.assertEqual(selector.count(".cradmin-legacy-roleselect-role-description"), 2)
                titletextlist = [
                    element.alltext_normalized
                    for element in selector.list(".cradmin-legacy-roleselect-role-description")
                ]
                self.assertEqual(titletextlist, ["Role One desc", "Role Two desc"])

    def test_render_list_urls(self):
        role1 = mock.MagicMock()
        role1.id = 1
        role1.title = "Role One"
        role2 = mock.MagicMock()
        role2.id = 2
        role2.title = "Role Two"
        cradmin_instance = self.__mock_cradmin_instance(roles=[role1, role2])

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch("cradmin_legacy.views.roleselect.cradmin_instance_registry", cradmin_instance_registry):
            with mock.patch(
                "cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry", cradmin_instance_registry
            ):
                request = self.factory.get("/roleselecttest")
                response = RoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)

                urllist = [element["href"] for element in selector.list(".cradmin-legacy-roleselect-list li a")]
                self.assertEqual(urllist, ["/role/1", "/role/2"])

    def test_render_pagination(self):
        class CustomRoleSelectView(RoleSelectView):
            paginate_by = 3

        roles = []
        for index in range(CustomRoleSelectView.paginate_by + 2):
            role = mock.MagicMock()
            role.id = index
            role.title = f"Role {index}"
            roles.append(role)
        cradmin_instance = self.__mock_cradmin_instance(roles=roles)

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch("cradmin_legacy.views.roleselect.cradmin_instance_registry", cradmin_instance_registry):
            with mock.patch(
                "cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry", cradmin_instance_registry
            ):
                request_page1 = self.factory.get("/roleselecttest")
                response_page1 = CustomRoleSelectView.as_view()(request_page1)
                self.assertEqual(response_page1.status_code, 200)
                response_page1.render()
                selector_page1 = htmls.S(response_page1.content)

                request_page2 = self.factory.get("/roleselecttest", {"page": 2})
                response_page2 = CustomRoleSelectView.as_view()(request_page2)
                self.assertEqual(response_page2.status_code, 200)
                response_page2.render()
                selector_page2 = htmls.S(response_page2.content)

                self.assertEqual(selector_page1.count(".cradmin-legacy-roleselect-list li"), 3)
                self.assertEqual(selector_page2.count(".cradmin-legacy-roleselect-list li"), 2)

                self.assertTrue(selector_page1.exists(".pager-container"))
                self.assertTrue(selector_page1.exists(".pager-container li.previous.disabled"))
                self.assertFalse(selector_page1.exists(".pager-container li.next.disabled"))
                self.assertTrue(selector_page2.exists(".pager-container"))
                self.assertFalse(selector_page2.exists(".pager-container li.previous.disabled"))
                self.assertTrue(selector_page2.exists(".pager-container li.next.disabled"))

    def test_render_no_pagination(self):
        class CustomRoleSelectView(RoleSelectView):
            paginate_by = 3

        roles = []
        for index in range(CustomRoleSelectView.paginate_by):
            role = mock.MagicMock()
            role.id = index
            role.title = f"Role {index}"
            roles.append(role)
        cradmin_instance = self.__mock_cradmin_instance(roles=roles)

        cradmin_instance_registry = mock.MagicMock()
        cradmin_instance_registry.get_current_instance.return_value = cradmin_instance
        with mock.patch("cradmin_legacy.views.roleselect.cradmin_instance_registry", cradmin_instance_registry):
            with mock.patch(
                "cradmin_legacy.templatetags.cradmin_legacy_tags.cradmin_instance_registry", cradmin_instance_registry
            ):
                request = self.factory.get("/roleselecttest")
                response = CustomRoleSelectView.as_view()(request)
                self.assertEqual(response.status_code, 200)
                response.render()
                selector = htmls.S(response.content)

                # selector.one('#cradmin_legacy_roleselect').prettyprint()
                self.assertEqual(selector.count(".cradmin-legacy-roleselect-list li"), 3)
                self.assertFalse(selector.exists(".pager-container"))



from django.contrib import messages
from django.utils.deprecation import MiddlewareMixin


class MessagesDebugMiddleware(MiddlewareMixin):
    """
    Add ``cradmin_legacy.messages_debug_middleware.MessagesDebugMiddleware``
    to your MIDDLEWARE setting to debug Django messages
    rendering/styling. Will add one of each message type to the request.
    """

    def process_request(self, request):
        messages.debug(request, "A debug message.")
        messages.info(request, "An info message.")
        messages.success(request, "A success  message.")
        messages.warning(request, "A warning message.")
        messages.error(request, "An error message.")



import htmls
from django.test import TestCase

from cradmin_legacy.python2_compatibility import mock
from cradmin_legacy.viewhelpers.listfilter.base.abstractfilterlist import AbstractFilterList
from cradmin_legacy.viewhelpers.listfilter.basefilters.single.abstractradio import AbstractRadioFilter


class TestAbstractRadioFilter(TestCase):
    def test_render_sanity(self):
        class MyRadioFilter(AbstractRadioFilter):
            def get_choices(self):
                return [
                    ("first", "First choice"),
                    ("second", "Second choice"),
                    ("third", "Third choice"),
                ]

        testfilter = MyRadioFilter(slug="test")
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertTrue(selector.exists("#cradmin_legacy_listfilter_test.cradmin-legacy-listfilter-filter"))
        self.assertTrue(selector.exists("#cradmin_legacy_listfilter_test_input"))
        self.assertEqual(3, selector.count('input[type="radio"]'))

    def test_render_add_value(self):
        class MyRadioFilter(AbstractRadioFilter):
            def get_choices(self):
                return [
                    ("first", "First choice"),
                ]

        testfilter = MyRadioFilter(slug="test")
        testfilter.set_values(values=[])
        filterlist = AbstractFilterList(
            urlbuilder=lambda filters_string: f"/test/{filters_string}", target_dom_id="testdomid"
        )
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual("/test/test-first", selector.one('input[type="radio"]')["data-url"])

    def test_render_radio_label(self):
        class MyRadioFilter(AbstractRadioFilter):
            def get_choices(self):
                return [
                    ("somevalue", "Test"),
                ]

        testfilter = MyRadioFilter(slug="test")
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual(
            "Test", selector.one("#cradmin_legacy_listfilter_test_input_somevalue_wrapper").alltext_normalized
        )

    def test_render_radio_unchecked(self):
        class MyRadioFilter(AbstractRadioFilter):
            def get_choices(self):
                return [
                    ("somevalue", "Test"),
                ]

        testfilter = MyRadioFilter(slug="test")
        testfilter.set_values(values=[])
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertFalse(selector.one('input[type="radio"]').hasattribute("checked"))

    def test_render_radio_checked(self):
        class MyRadioFilter(AbstractRadioFilter):
            def get_choices(self):
                return [
                    ("somevalue", "Test"),
                ]

        testfilter = MyRadioFilter(slug="test")
        testfilter.set_values(values=["somevalue"])
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertTrue(selector.one('input[type="radio"]').hasattribute("checked"))

    def test_render_label_no_label(self):
        class MyRadioFilter(AbstractRadioFilter):
            def get_choices(self):
                return [
                    ("unused", "unused"),
                ]

        testfilter = MyRadioFilter(slug="test")
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertFalse(selector.exists("label#cradmin_legacy_listfilter_test_label"))

    def test_render_label_has_label(self):
        class MyRadioFilter(AbstractRadioFilter):
            def get_choices(self):
                return [
                    ("unused", "unused"),
                ]

        testfilter = MyRadioFilter(slug="test", label="A label")
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual("A label", selector.one("label#cradmin_legacy_listfilter_test_label").alltext_normalized)

    def test_render_label_get_label_is_screenreader_only_true(self):
        class MyRadioFilter(AbstractRadioFilter):
            def get_choices(self):
                return [
                    ("unused", "unused"),
                ]

            def get_label_is_screenreader_only(self):
                return True

        testfilter = MyRadioFilter(slug="test", label="A label")
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertTrue(selector.exists("label#cradmin_legacy_listfilter_test_label"))
        self.assertTrue(selector.one("label#cradmin_legacy_listfilter_test_label").hasclass("sr-only"))



from django.urls import path

from cradmin_legacy.apps.cradmin_authenticate.views.login import LoginView
from cradmin_legacy.apps.cradmin_authenticate.views.logout import cradmin_logoutview

urlpatterns = [
    path("login", LoginView.as_view(), name="cradmin-authenticate-login"),
    path("logout", cradmin_logoutview, name="cradmin-authenticate-logout"),
]



import logging
from unittest import mock

import htmls
from django.test import TestCase
from model_bakery import baker

from cradmin_legacy.tests.viewhelpers.cradmin_viewhelpers_testapp.models import (
    FilterTestModel,
)
from cradmin_legacy.viewhelpers.listfilter.base import filtershandler
from cradmin_legacy.viewhelpers.listfilter.base.abstractfilter import AbstractFilter
from cradmin_legacy.viewhelpers.listfilter.base.abstractfilterlist import (
    AbstractFilterList,
)
from cradmin_legacy.viewhelpers.listfilter.base.abstractfilterlistchild import (
    AbstractFilterListChild,
)


class MinimalFilterGroupChild(AbstractFilterListChild):
    template_name = "cradmin_viewhelpers_testapp/listfilter/minimal-filtergroupchild.django.html"


class MinimalIntFilter(AbstractFilter):
    template_name = "cradmin_viewhelpers_testapp/listfilter/minimal-filtergroupchild.django.html"

    def get_slug(self):
        return "i"

    def clean_value(self, value):
        return int(value)


class MinimalStringFilter(AbstractFilter):
    template_name = "cradmin_viewhelpers_testapp/listfilter/minimal-filtergroupchild.django.html"

    def get_slug(self):
        return "s"


class TestAbstractFilterList(TestCase):
    def test_append(self):
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        testchild = MinimalFilterGroupChild()
        filterlist.append(testchild)
        self.assertEqual([testchild], filterlist.children)
        self.assertEqual(filterlist, testchild.filterlist)

    def test_render(self):
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(MinimalFilterGroupChild())
        filterlist.append(MinimalFilterGroupChild())
        selector = htmls.S(filterlist.render(request=mock.MagicMock()))
        self.assertEqual(2, selector.count("li"))

    def test_set_filters_string_invalid_slug(self):
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        stringfilter = MinimalStringFilter()
        filterlist.append(stringfilter)
        with self.assertLogs(filtershandler.logger, level=logging.DEBUG) as logcontext:
            filterlist.set_filters_string("x-10")
            self.assertEqual(len(logcontext.records), 1)
            self.assertEqual(logcontext.records[0].message, '"x" is not a valid filter slug.')

    def test_set_filters_string(self):
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        intfilter = MinimalIntFilter()
        stringfilter = MinimalStringFilter()
        filterlist.append(intfilter)
        filterlist.append(stringfilter)
        with self.assertNoLogs(filtershandler.logger, level=logging.DEBUG):
            filterlist.set_filters_string("i-10/s-test")
        self.assertEqual(["10"], intfilter.values)
        self.assertEqual(["test"], stringfilter.values)

    def test_filter(self):
        class FilterOne(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mycharfield="test")

        class FilterTwo(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mybooleanfield=True)

        match = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="test", mybooleanfield=True)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="no match", mybooleanfield=True)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="test", mybooleanfield=False)

        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(FilterOne(slug="filterone"))
        filterlist.append(FilterTwo(slug="filtertwo"))
        queryset = filterlist.filter(FilterTestModel.objects.all())
        self.assertEqual({match}, set(queryset))

    def test_filter_exclude(self):
        class FilterOne(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mycharfield="test")

        class FilterTwo(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mybooleanfield=True)

        match1 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="test", mybooleanfield=True)
        match2 = baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="no match", mybooleanfield=True)
        baker.make("cradmin_viewhelpers_testapp.FilterTestModel", mycharfield="test", mybooleanfield=False)

        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id="testdomid")
        filterlist.append(FilterOne(slug="filterone"))
        filterlist.append(FilterTwo(slug="filtertwo"))
        queryset = filterlist.filter(FilterTestModel.objects.all(), exclude={"filterone"})
        self.assertEqual({match1, match2}, set(queryset))



from django.core import mail
from django.urls import reverse
from django.test import TestCase
import htmls
from cradmin_legacy.python2_compatibility import mock

from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from cradmin_legacy.apps.cradmin_resetpassword.views.begin import BeginPasswordResetView
from cradmin_legacy.tests.helpers import create_user


class TestBeginPasswordResetView(TestCase):
    def setUp(self):
        self.url = reverse("cradmin-resetpassword-begin")

    def test_get(self):
        response = self.client.get(self.url)
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists("form#cradmin_legacy_resetpassword_begin_form"))
        self.assertEqual(selector.one("h1").alltext_normalized, "Find your account")
        self.assertIn("Type in your email-address", selector.one(".page-header").alltext_normalized)
        self.assertTrue(selector.exists('input[type="email"][name="email"]'))
        self.assertEqual(selector.one('button[type="submit"]').alltext_normalized, "Search")

    def test_post_user_not_found(self):
        response = self.client.post(self.url, {"email": "testuser@example.com"})
        self.assertEqual(response.status_code, 200)
        selector = htmls.S(response.content)
        self.assertIn(
            "No account with this email address found",
            selector.one("form#cradmin_legacy_resetpassword_begin_form").alltext_normalized,
        )

    def test_post_user_found(self):
        create_user("testuser", email="testuser@example.com")
        with mock.patch.object(BeginPasswordResetView, "_generate_token", lambda s, user: "testtoken"):
            with self.settings(CRADMIN_LEGACY_SITENAME="Testsite"):
                response = self.client.post(self.url, {"email": "testuser@example.com"})
        self.assertRedirects(response, reverse("cradmin-resetpassword-email-sent"))

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "Reset your Testsite password")
        self.assertIn("http://testserver/cradmin_resetpassword/reset/testtoken", mail.outbox[0].alternatives[0][0])
        self.assertIn(
            "We received a request to reset the password for your Testsite account, testuser",
            mail.outbox[0].alternatives[0][0],
        )

    def test_post_token_created(self):
        create_user("testuser", email="testuser@example.com")
        self.assertEqual(GenericTokenWithMetadata.objects.count(), 0)
        self.client.post(self.url, {"email": "testuser@example.com"})
        self.assertEqual(GenericTokenWithMetadata.objects.count(), 1)
        token = GenericTokenWithMetadata.objects.first()
        self.assertIsNotNone(token.expiration_datetime)
